package hero.entity;

import hero.interfaces.BnEdgePK;
import hero.interfaces.BnEdgeValue;
import hero.interfaces.BnEdgeLightValue;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.InvalidValueException;
import hero.interfaces.SequenceGenerator;
import hero.interfaces.SequenceGeneratorHome;
import hero.interfaces.ServiceUnavailableException;

import java.rmi.RemoteException;
import java.sql.Date;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

/**
 * The Entity bean represents a BnEdge
 *
 * @author Franois Charoy
 * @version $Revision: 0.1 $
 *
 * @ejb:bean name="BnEdge"
 *           display-name="BnEdge Entity"
 *           type="CMP"
 *           jndi-name="ejb/hero/BnEdge"
 *           local-jndi-name="ejb/hero/BnEdge_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"   
 *
 * @ejb:env-entry name="SequenceName"
 *                value="BnEdge"
 *
 * @ejb:ejb-ref ejb-name="hero/SequenceGenerator"
 *
 * @ejb:transaction type="Supports"
 *
 * @jonas.bean
 *     ejb-name="BnEdge"
 *     jndi-name="ejb/hero/BnEdge"
 * @jonas.jdbc-mapping
 *     jndi-name="jdbc_1"
 *     jdbc-table-name="BnEdge"
 * @jonas.cleanup create
 *
 *
 * @ejb:finder signature="java.util.Collection findAll()" 
 *     query="SELECT object(a) FROM BnEdge a"
 *     transaction-type="Supports"
 *
 * @ejb:finder signature="hero.interfaces.BnEdge findByName(java.lang.String pName, int projectId )" view-type="both" result-type-mapping="Remote" 
 *     query="SELECT OBJECT(n) FROM BnEdge n WHERE n.name = ?1 AND n.bnProject.id = ?2"
 *     transaction-type="Supports"
 * @ejb:finder signature="hero.interfaces.BnEdgeLocal findByName(java.lang.String pName, int projectId )" view-type="both" result-type-mapping="Local" 
 *     query="SELECT OBJECT(e) FROM BnEdge e WHERE e.name = ?1 AND e.bnProject.id = ?2"
 *     transaction-type="Supports"
 *
 * @ejb.value-object
 *    match="*"
 *    name="BnEdge"
 * 
  * @ejb.value-object
 *    match="light"
 *    name="BnEdgeLight"
 *
 * @jboss:table-name table-name="BnEdge"
 *
 * @jboss:create-table create="true"
 *
 * @jboss:remove-table remove="false"
 *
 * @jonas.ejb-relation
 *        ejb-relation-name="InBnNode"
 *        ejb-relationship-role-name1="has-node-in"
 *        foreign-key-jdbc-name1="has_node_edge"
 *        key-jdbc-name1="id"
 **/
public abstract class BnEdgeBean
   implements EntityBean
{
   
   // -------------------------------------------------------------------------
   // Members
   // -------------------------------------------------------------------------  
   
   public EntityContext mContext;
   
   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------     
   
   /**
   * Retrive a unique creation id to use for this bean.  This will end up
   * demarcating this bean from others when it is stored as a record
   * in the database.
   *
   * @return Returns an integer that can be used as a unique creation id.
   *
   * @throws ServiceUnavailableException Indicating that it was not possible
   *                                     to retrieve a new unqiue ID because
   *                                     the service is not available
   **/
   private int generateUniqueId()
      throws ServiceUnavailableException
   {
      int lUniqueId = -1;
      try {
         Context lContext = new InitialContext();
         
         String lSequenceName = (String) lContext.lookup( 
            "java:comp/env/SequenceName" 
         );
         SequenceGeneratorHome lHome = (SequenceGeneratorHome) PortableRemoteObject.narrow(
            lContext.lookup(
               "java:comp/env/ejb/hero/SequenceGenerator"
            ),
            SequenceGeneratorHome.class
         );
         SequenceGenerator lBean = (SequenceGenerator) lHome.create();
         lUniqueId = lBean.getNextNumber( lSequenceName );
         lBean.remove();  
      }
      catch ( NamingException ne ) {
         throw new ServiceUnavailableException( "Naming lookup failure: " + ne.getMessage() );
      }
      catch ( CreateException ce ) {
         throw new ServiceUnavailableException( "Failure while creating a generator session bean: " + ce.getMessage() );
      }
      catch ( RemoveException re ) {
         // When the Bean cannot be removed after a while it will be taken back by the container
         // therefore ignore this exception
      }
      catch ( RemoteException rte ) {
         throw new ServiceUnavailableException( "Remote exception occured while accessing generator session bean: " +  rte.getMessage() );
      }
      
      return lUniqueId;
   }
   
   // -------------------------------------------------------------------------
   // Properties (Getters/Setters)
   // -------------------------------------------------------------------------  
   
   /**
   * Retrieve the BnEdge's id.
   *
   * @return Returns an int representing the id of this BnEdge.
   *
   * @ejb.value-object
   *    match="light"   
   * @ejb:pk-field
   *
   * @ejb:persistence column-name="id"
   * @jonas.cmp-field-jdbc-mapping
   *    field-name="id"
   *    jdbc-field-name="id"
   * @ejb:transaction type="Supports"
   **/
   public abstract int getId();
   
   /**
   * Set the BnEdge's id.
   *
   * @param pId The id of this BnEdge. Is set at creation time.
   * @ejb:transaction type="Required"
  **/
   public abstract void setId( int pId );
   
   /**
   * Retrieve the BnEdge's name.
   *
   * @return Returns a string representing the name of this BnEdge.
   * @ejb.value-object
   *    match="light"   
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="name"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getName();
   
   /**
   * Set the BnEdge's name.
   *
   * @param pName The name of this BnEdge. Is set at creation time.
   * @ejb:transaction type="Required"
  **/
   public abstract void setName( String pName );

    /**
     * @return  the project of the BnEdge 
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="Edges" role-name="is-edge-of" target-ejb="BnProject" cascade-delete="yes"
     * @jboss:auto-key-fields 
     * @jonas:automatic-pk="true"
     * @ejb:transaction type="Supports"
     *
     **/
    public abstract BnProjectLocal getBnProject();
    
    /**
     * Set the BnProject Name.
     *
     * @param pName The Name of this BnEdge.  Is set at creation time.
     * @ejb:interface-method view-type="local"
     * @ejb:transaction type="Required"
     **/
    public abstract void setBnProject(BnProjectLocal pProject);
    
    /**
     * @ejb.value-object
     *    match="light"   
     * @return  the state of the BnEdge
     * @ejb:interface-method view-type="local"
     * @ejb:persistence column-name="state"
     * @ejb:transaction type="Supports"
     *
     **/
    public abstract int getState();
    
    /**
     * Set the State value.
     *
     * @param pstate The new state of this BnEdge. 
     * @ejb:interface-method view-type="local"
     * @ejb:transaction type="Required"
     **/
    public abstract void setState(int state);

    /**
     * @ejb.value-object
     *    match="light"    
     * @return  the condition of the BnEdge
     * @ejb:interface-method view-type="local"
     * @ejb:persistence column-name="condition"
     * @ejb:transaction type="Supports"
     *
     **/
    public abstract String getCondition();
    
    /**
     * Set the Condition value.
     *
     * @param condition The new state of this BnEdge. 
     * @ejb:interface-method view-type="local"
     * @ejb:transaction type="Required"
     **/
    public abstract void setCondition(String condition);

    /**
     * @return  the In BnNode of the BnEdge
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="InBnNode" role-name="has-node-in" target-ejb="BnNode"
     * @ejb:value-object match="*"
     *       relation="external"
     *       aggregate="hero.interfaces.BnNodeValue"
     *       aggregate-name="InBnNode"
     * @jboss:auto-key-fields
     * @jonas:automatic-pk="true"
     * @ejb:transaction type="Supports"
     *
     **/
    public abstract BnNodeLocal getInBnNode();
    
   /**
   * Set the BnProject Name.
   *
   * @param pNode The Name of this BnEdge.  Is set at creation time.
   * @ejb:interface-method view-type="local"
   * @ejb:transaction type="Required"
   **/
    public abstract void setInBnNode(BnNodeLocal pNode);

    /**
     * @return  the Out BnNode of the BnEdge
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="OutBnNode" role-name="has-node-out" target-ejb="BnNode"
     * @ejb:value-object match="*" 
     *       relation="external"
     *       aggregate="hero.interfaces.BnNodeValue"
     *       aggregate-name="OutBnNode"     
     * @jboss:auto-key-fields 
     * @jonas:automatic-pk="true"
     * @ejb:transaction type="Supports"
     *
     **/
    public abstract BnNodeLocal getOutBnNode();

   /**
   * Set the BnProject Name.
   *
   * @param pNode The Name of this BnEdge.  Is set at creation time.
   * @ejb:interface-method view-type="local"
   * @ejb:transaction type="Required"
   **/
    public abstract void setOutBnNode(BnNodeLocal pNode);
    /**
     * @return Return the creation date of the BnEdge
     * @ejb:interface-method view-type="both"
     * @ejb:persistence column-name="creationdate"
     * @ejb:transaction type="Supports"
     *
     **/

    public abstract Date getCreationDate();

    /**
     * @param pDate the creation date of the BnEdge
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Required"
     *
     **/
    public abstract void setCreationDate(Date pDate);

    /**
     * @ejb.value-object
     *    match="light"   
     * @return Return the modification date of the BnEdge
     * @ejb:interface-method view-type="both"
     * @ejb:persistence column-name="modificationdate"
     * @ejb:transaction type="Supports"
     *
     **/
    public abstract Date getModificationDate();

    /**
     * @param pDate the creation date of the BnEdge
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Required"
     *
     **/
    public abstract void setModificationDate(Date pDate);

    /**
     * @ejb.interface-method 
     * @ejb:transaction type="Supports"
     */
    public abstract BnEdgeValue getBnEdgeValue();
    
    /**
     * @ejb.interface-method 
     * @ejb:transaction type="Required"
     */
    public abstract void setBnEdgeValue(BnEdgeValue v);
    
	/**
	  * @ejb.interface-method 
	  * @ejb:transaction type="Supports"
	  */
	 public abstract BnEdgeLightValue getBnEdgeLightValue();
    
	 /**
	  * @ejb.interface-method 
	  * @ejb:transaction type="Required"
	  */
	 public abstract void setBnEdgeLightValue(BnEdgeLightValue v);
   
   // -------------------------------------------------------------------------
   // Framework Callbacks
   // -------------------------------------------------------------------------  
   
   /**
   * Create a BnEdge based on the supplied BnEdge Value Object.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnEdgePK ejbCreate()
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
      try {
         // Each title must have a unique id to identify itself within the DB
         this.setId( generateUniqueId() );
	 this.setName("edge"+this.getId());
      }
      catch( ServiceUnavailableException se ) {
         // The unique id could not be set therefore terminate the transaction
         // by throwing a system exception
         throw new EJBException( se.getMessage() );
      }
      return null;
   }

   /**
   * Create a BnEdge based on the supplied BnEdge Value Object. (c'est merdique)
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnEdgePK ejbCreate(BnEdgeValue pEdge)
      throws
         CreateException
   {
       this.setBnEdgeValue(pEdge);
       return null;
   }

   
   public void ejbPostCreate( )
   {
   }
   
   public void setEntityContext( EntityContext lContext )
   {
      mContext = lContext;
   }
   
   public void unsetEntityContext()
   {
      mContext = null;
   }
   
   public void ejbActivate()
   {
   }
   
   public void ejbPassivate()
   {
   }
   
   public void ejbLoad()
   {
   }
   
   public void ejbStore()
   {
   }
   
   public void ejbRemove()
      throws
         RemoveException
   {
   }

}
