package hero.entity;

import hero.interfaces.BnNodeLightValue;
import hero.interfaces.BnNodePK;
import hero.interfaces.BnNodeValue;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnRoleLocal;
import hero.interfaces.BnUserLocal;
import hero.interfaces.InvalidValueException;
import hero.interfaces.SequenceGenerator;
import hero.interfaces.SequenceGeneratorHome;
import hero.interfaces.ServiceUnavailableException;

import java.rmi.RemoteException;
import java.util.Date;
import java.util.Collection;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

/**
 * The Entity bean represents a BnUser
 *
 * @author Fran?s Charoy
 * @version $Revision: 0.1 $
 *
 * @ejb:bean name="BnNode"
 *           display-name="BnNode Entity"
 *           type="CMP"
 *           jndi-name="ejb/hero/BnNode"
 *           local-jndi-name="ejb/hero/BnNode_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"   
 *
 * @ejb:env-entry name="SequenceName"
 *                value="BnNode"
 *
 * @ejb:ejb-ref ejb-name="hero/SequenceGenerator"
 *
 * @ejb:transaction type="Supports"
 *
 * @ejb.pk method-intf="both"
 * 
 * @jonas.bean
 *     ejb-name="BnNode"
 *     jndi-name="ejb/hero/BnNode"
 * @jonas.jdbc-mapping
 *     jndi-name="jdbc_1"
 *     jdbc-table-name="BnNode"
 * @jonas.cleanup create
 *
 * @ejb:finder signature="java.util.Collection findAll()" 
 *     query="SELECT object(n) FROM BnNode n"
 *     transaction-type="Supports"
 *
 * @ejb:finder signature="hero.interfaces.BnNode findByName(java.lang.String pName, int projectId )" view-type="both" result-type-mapping="Remote" 
 *     query="SELECT OBJECT(n) FROM BnNode n WHERE n.name = ?1 AND n.bnProject.id = ?2"
 *     transaction-type="Supports"
 * @ejb:finder signature="hero.interfaces.BnNodeLocal findByName(java.lang.String pName, int projectId )" view-type="both" result-type-mapping="Local" 
 *     query="SELECT OBJECT(n) FROM BnNode n WHERE n.name = ?1 AND n.bnProject.id = ?2"
 *     transaction-type="Supports"
 *
 * @ejb.finder 
 *     signature="Collection findUserReady(int projectId, java.lang.String userName)" 
 *     view-type="local" result-type-mapping="Local" 
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnProject AS p, IN (p.bnUsers) AS u, IN (u.bnRoles) AS r
 *                              WHERE n.bnProject.id = ?1 
 *                              AND (n.state = 1 OR n.state =3)
 *                              AND u.name = ?2
 *                              AND r.id = n.bnRole.id"
 *    transaction-type="Supports" 
 *
 * @ejb.finder 
 *     signature="Collection findUserExecuting(int projectId, java.lang.String userName)" 
 *     view-type="local" result-type-mapping="Local" 
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n
 *                              WHERE n.bnProject.id = ?1 
 *                              AND (n.state = 5 OR n.state =6)
 *                              AND n.executor.name = ?2"
 *     transaction-type="Supports"
 *
 * @ejb.value-object
 *    match="*"
 *    name="BnNode"
 *
 * @ejb.value-object
 *    match="light"
 *    name="BnNodeLight"
 *
 * @jboss:table-name table-name="BnNode"
 *
 * @jboss:create-table create="true"
 *
 * @jboss:remove-table remove="false"
 * 
 * @jonas.ejb-relation
 *        ejb-relation-name="Executor"
 *        ejb-relationship-role-name1="executed-by"
 *        foreign-key-jdbc-name1="executor_node"
 *        key-jdbc-name1="id" 
 **/
public abstract class BnNodeBean
   implements EntityBean
{
   
   // -------------------------------------------------------------------------
   // Members
   // -------------------------------------------------------------------------  
   
   public EntityContext mContext;
   
   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------     
   

   // -------------------------------------------------------------------------
   // Properties (Getters/Setters)
   // -------------------------------------------------------------------------  
   
   /**
   * Retrieve the BnNode's id.
   *
   * @return Returns an int representing the id of this BnNode.
   *
   * @ejb:pk-field
   * @ejb.value-object
   *    match="light"   
   *
   * @ejb:interface-method view-type="local"
   * @ejb:persistence column-name="id"
   * @jonas.cmp-field-jdbc-mapping
   *    field-name="id"
   *    jdbc-field-name="id"
   * @ejb:transaction type="Supports"
   **/
   public abstract int getId();
   
   /**
   * Set the BnNode's id.
   *
   * @param pId The id of this BnNode. Is set at creation time.
   * @ejb:transaction type="Required"
   **/
   public abstract void setId( int pId );
   
   /**
   * Retrieve the BnNode's type.
   *
   * @return Returns an int representing the type of this BnNode.
   *
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"   
   *
   * @ejb:persistence column-name="type"
   * @ejb:transaction type="Supports"
   **/
   public abstract int getType();
   
   /**
   * Set the BnNode's type.
   *
   * @param pType The type of this BnNode. Is set at creation time.
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Required"
   **/
   public abstract void setType( int pType );

   /**
   * Retrieve the BnNode's state.
   *
   * @return Returns an int representing the state of this BnNode.
   *
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"   
   *
   * @ejb:persistence column-name="state"
   * @ejb:transaction type="Supports"
   **/
   public abstract int getState();
   
   /**
   * Set the BnNode's state.
   *
   * @param pState The type of this BnNode. Is set at creation time.
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Required"
   **/
   public abstract void setState( int pState );

   /**
   * Retrieve the BnNode's anticipation mode.
   *
   * @return Returns a boolean indicating if the node is anticipable
   *
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"   
   *
   * @ejb:persistence column-name="anticipable"
   * @ejb:transaction type="Supports"
   **/
   public abstract boolean getAnticipable();
   
   /**
   * Set the BnNode's Anticipable mode.
   *
   * @param pAnticipate The mode of this BnNode. 
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Required"
   **/
   public abstract void setAnticipable( boolean pAnticipable);

   /**
   * Retrieve the BnNode's Name.
   *
   * @return Returns an int representing the Name of this BnNode.
   *
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"   
   *
   * @ejb:persistence column-name="name"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getName();
   
   /**
   * Set the BnNode's Name.
   *
   * @param pName The Name of this BnNode.  Is set at creation time.
   * @ejb:transaction type="Required"
   **/
   public abstract void setName( String pName );

   /**
   * Retrieve the BnNode's Description.
   *
   * @return Returns the description of the node
   *
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"   
   *
   * @ejb:persistence column-name="description"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getDescription();
   
   /**
   * Set the BnNode's Description.
   *
   * @param pDescription The Description of this BnNode.
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Required"
   **/
   public abstract void setDescription( String pDescription );

   /**
   * Retrieve the BnNode's Transition type.
   *
   * @return Returns the description of the node
   *
   * @ejb:interface-method view-type="local"
   *
   * @ejb:persistence column-name="transition"
   * @ejb:transaction type="Supports"
   **/
   public abstract NodeState getTransition();
   
   /**
   * Set the BnNode's Transition
   *
   * @param transition The Transition of this BnNode.
   * @ejb:interface-method view-type="local"
   * @ejb:transaction type="Required"
   **/
   public abstract void setTransition( NodeState transition );


   /**
   * Retrieve the BnNode's Activation type.
   *
   * @return Returns the description of the node
   *
   * @ejb:interface-method view-type="local"
   *
   * @ejb:persistence column-name="activation"
   * @ejb:transaction type="Supports"
   **/
   public abstract EdgeState getActivation();
   
   /**
   * Set the BnNode's Activation
   *
   * @param activation The Activation of this BnNode.
   * @ejb:interface-method view-type="local"
   * @ejb:transaction type="Required"
   **/
   public abstract void setActivation( EdgeState activation );

   /**
   * Retrieve the BnNode's StartDate.
   *
   * @return Returns the start date of the node
   *
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"   
   *
   * @ejb:persistence column-name="startdate"
   * @ejb:transaction type="Supports"
   **/
   public abstract Date getStartDate();
   
   /**
   * Set the BnNode's StartDate.
   *
   * @param pStartDate The StartDate of this BnNode.
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Required"
   **/
   public abstract void setStartDate( Date pStartDate );

   /**
   * Retrieve the BnNode's EndDate.
   *
   * @return Returns the end date of the node
   *
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"   
   *
   * @ejb:persistence column-name="enddate"
   * @ejb:transaction type="Supports"
   **/
   public abstract Date getEndDate();
   
   /**
   * Set the BnNode's EndDate.
   *
   * @param pEndDate The end date of this BnNode.
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Required"
   **/
   public abstract void setEndDate( Date pEndDate );

   /**
   * Retrieve the BnNode's Deadline.
   *
   * @return Returns the deadline of the node
   *
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"   
   *
   * @ejb:persistence column-name="deadline"
   * @ejb:transaction type="Supports"
   **/
   public abstract Date getDeadline();
   
   /**
   * Set the BnNode's Deadline.
   *
   * @param pDeadline The Deadline of this BnNode.
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Required"
   **/
   public abstract void setDeadline( Date pDeadline );

    /** Retrieve the BnNode's Creator.
     *
     * @return Returns a String representing the Name of the BnNode Creator.
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="Creator" role-name="created-by"
     * @jboss:auto-key-fields
     * @jonas:automatic-pk="true"
	 *
     * @ejb:value-object match="*" 
     *       relation="external"
     *       aggregate="hero.interfaces.BnUserLightValue"
     *       aggregate-name="Creator"
     * @ejb:transaction type="Supports"
     **/
    public abstract BnUserLocal getCreator();
    
    /**
     * Set the BnNode's Creator.
     *
     * @param pCreator The creator of this BnNode.
     * @ejb:interface-method view-type="local"
     * @ejb:transaction type="Required"
     **/
    public abstract void setCreator( BnUserLocal pCreator );
    
    /** Retrieve the BnNode's Executor.
     *
     * @return Returns a String representing the Name of the BnNode Creator.
     * @ejb:interface-method view-type="both"
     * @ejb:relation name="Executor" role-name="executed-by"
     * @jboss:auto-key-fields
     * @ejb:value-object match="*" 
     *       relation="external"
     *       aggregate="hero.interfaces.BnUserLightValue"
     *       aggregate-name="Executor"
     * @ejb:transaction type="Supports"
     **/
    public abstract BnUserLocal getExecutor();
    
    /**
     * Set the BnNode's Creator.
     *
     * @ejb:interface-method view-type="local"
     * @param pCreator The creator of this BnNode.  Is set at creation time.
     * @ejb:transaction type="Required"	
     **/
    public abstract void setExecutor( BnUserLocal pExecutor );

    /** Retrieve the BnNode's BnRole.
     *
     * @return Returns a String representing the ole associated to the BnNode.
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="NodeRole" role-name="has-role"
     * @ejb:value-object match="*" 
     *       relation="external"
     *       aggregate="hero.interfaces.BnRoleValue"
     *       aggregate-name="BnRole"     
     * @jboss:auto-key-fields
     * @jonas:automatic-pk="true"
     * @ejb:transaction type="Supports"
     **/
    public abstract BnRoleLocal getBnRole();
    
    /**
     * Set the BnNode's Creator.
     *
     * @param pCreator The creator of this BnNode.  Is set at creation time.
     * @ejb:interface-method view-type="local"
     * @ejb:transaction type="Required"
     **/
    public abstract void setBnRole(BnRoleLocal role);


    /**
     * @return  the project of the BnNode
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="Nodes" role-name="is-node-of" cascade-delete="yes"
     * @jboss:auto-key-fields 
     * @jonas:automatic-pk="true"
     * @ejb:transaction type="Supports"
     *
     **/
    public abstract BnProjectLocal getBnProject();

   /**
   * Set the BnProject Name.
   *
   * @param pName The Name of this BnNode.  Is set at creation time.
   * @ejb:transaction type="Required"
   **/
    public abstract void setBnProject(BnProjectLocal pProject);

    /**
     * @return  the outgoing edges of the node
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="InBnNode" role-name="has-edges-in" target-multiple="yes"
     * @jboss:auto-key-fields 
     * @jonas:automatic-pk="true"
     * @ejb:transaction type="Supports"
     *
     **/
    public abstract Collection getOutBnEdges();

   /**
   * Set the Outgoing edges of the node
   *
   * @param pEdges the outgoing edges of the node
   * @ejb:transaction type="Required"
   **/
    public abstract void setOutBnEdges(Collection pEdges);    

    /**
     * @return  the ingoing edges of the node
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="OutBnNode" role-name="has-edges-out" target-multiple="yes"
     * @jboss:auto-key-fields 
     * @jonas:automatic-pk="true"
     * @ejb:transaction type="Supports"
     *
     **/
    public abstract Collection getInBnEdges();

   /**
   * Set the Ingoing edges of the node
   *
   * @param pEdges the ingoing edges of the node
   * @ejb:transaction type="Required"
   **/
    public abstract void setInBnEdges(Collection pEdges);    
    
    /**
     * @return  the ingoing AgentEdges of the node
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="OutNodeInAgentEdge" role-name="has-edges-out" target-multiple="yes"
     * @jboss:auto-key-fields
     * @jonas:automatic-pk="true"
     * @ejb:transaction type="Supports"
     *
     **/
    public abstract Collection getInBnAgentEdges();
    
    /**
     * Set the Ingoing AgentEdges of the node
     *
     * @param pAgentEdges the ingoing AgentEdges of the node
     * @ejb:transaction type="Required"
     **/
    public abstract void setInBnAgentEdges(Collection pAgentEdges);    

    /**
     * @return Return the creation date of the BnNode
     * @ejb:interface-method view-type="both"
     * @ejb:persistence column-name="creationdate"
     * @ejb:transaction type="Supports"
     *
     **/
    public abstract Date getCreationDate();

    /**
     * @param pDate the creation date of the BnNode
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Required"
     *
     **/
    public abstract void setCreationDate(Date pDate);

    /**
     * @return Return the modification date of the BnNode
     * @ejb:interface-method view-type="both"
     * @ejb:persistence column-name="modificationdate"
     * @ejb:transaction type="Supports"
     *
     **/
    public abstract Date getModificationDate();

    /**
     * @param pDate the creation date of the BnNode
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Required"
     *
     **/
    public abstract void setModificationDate(Date pDate);


    /**
     * @return  the properties of the node
     * @ejb:interface-method view-type="both"
     * @ejb:relation name="nodeprp" role-name="is-property-of" 
     * @jboss:auto-key-fields 
     * @jonas:automatic-pk="true"
     * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       aggregate="hero.interfaces.BnNodePropertyValue"
	 *       aggregate-name="BnPropertie"
	 *       members="hero.interfaces.BnNodePropertyLocal"
	 *       members-name="BnPropertie"
     * @ejb:transaction type="Supports"
     *
     **/
    public abstract Collection getBnProperties();

   /**
   * Set the properties of the node
   * @ejb:interface-method view-type="both"
   * @param pPrp the outgoing edges of the node
   * @ejb:transaction type="Required"
   **/
    public abstract void setBnProperties(Collection pPrp);    

   /**
     * @return  the Hooks of the node
     * @ejb:interface-method view-type="both"
     * @ejb:relation name="nodehook" role-name="is-hook-of" 
     * @jboss:auto-key-fields 
     * @jonas:automatic-pk="true"
     * @ejb:transaction type="Supports"
     *
     **/
    public abstract Collection getBnHooks();

   /**
   * Set the Hooks of the node
   * @ejb:interface-method view-type="both"
   * @param pHook the outgoing edges of the node
   * @ejb:transaction type="Required"
   **/
    public abstract void setBnHooks(Collection pHook);    


   /**
     * @return  the Hooks of the node
     * @ejb:interface-method view-type="both"
     * @ejb:relation name="nodeinterhook" role-name="is-interhook-of" 
     * @jboss:auto-key-fields 
     * @jonas:automatic-pk="true"
     * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       aggregate="hero.interfaces.BnNodeInterHookValue"
	 *       aggregate-name="BnInterHook"
	 *       members="hero.interfaces.BnNodeInterHookLocal"
	 *       members-name="BnInterHook"
     * @ejb:transaction type="Supports"
     *
     **/
    public abstract Collection getBnInterHooks();

   /**
   * Set the Hooks of the node
   * @ejb:interface-method view-type="both"
   * @param pHook the outgoing edges of the node
   * @ejb:transaction type="Required"
   **/
    public abstract void setBnInterHooks(Collection pHook);    

    /**
     * @ejb.interface-method 
     * @ejb:transaction type="Supports"
     */
    public abstract BnNodeValue getBnNodeValue();
    
    /**
     * @ejb.interface-method 
     * @ejb:transaction type="Required"
     */
    public abstract void setBnNodeValue(BnNodeValue v);

    /**
     * @ejb.interface-method 
     * @ejb:transaction type="Supports"
     */
    public abstract BnNodeLightValue getBnNodeLightValue();
    
	/**
	 * @ejb.interface-method 
	 * @ejb:transaction type="Required"
	 */
	public abstract void setBnNodeLightValue(BnNodeLightValue v);
    

   /** 
     * @return true if the BnNode is executing
     * @ejb.interface-method view-type="local"
     * @ejb:transaction type="Supports"
     */ 
    public boolean isExecuting() { 
	return (this.getState()==hero.interfaces.Constants.Nd.EXECUTING || 
		this.getState()==hero.interfaces.Constants.Nd.ANTICIPATING); 
    } 

   /** 
     * @return true if the BnNode is TErminated
     * @ejb.interface-method view-type="local"
     * @ejb:transaction type="Supports"
     */ 
    public boolean isTerminated() { 
	return (this.getState()==hero.interfaces.Constants.Nd.TERMINATED || 
		this.getState()==hero.interfaces.Constants.Nd.FINISHED); 
    } 

   /** 
     * @return true if the BnNode is Cancelled
     * @ejb.interface-method view-type="local"
     * @ejb:transaction type="Supports"
     */ 
    public boolean isCancelled() { 
	return (this.getState()==hero.interfaces.Constants.Nd.DEAD);
    } 


    
   // -------------------------------------------------------------------------
   // Framework Callbacks
   // -------------------------------------------------------------------------  
   
   /**
   * Create a BnNode based on the supplied BnNode Value Object.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnNodePK ejbCreate(BnNodeValue nd )
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
       setBnNodeValue(nd);
      try {
         // Each title must have a unique id to identify itself within the DB
         this.setId( generateUniqueId() );
      }
      catch( ServiceUnavailableException se ) {
         // The unique id could not be set therefore terminate the transaction
         // by throwing a system exception
         throw new EJBException( se.getMessage() );
      }
      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   /**
   * Create a BnNode based on the supplied Name and type.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnNodePK ejbCreate(String name, int type )
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
       this.setName(name);
       this.setType(type);
       this.setState(hero.interfaces.Constants.Nd.READY);
       this.setAnticipable(true);
       this.setTransition(NodeState.make(type,true));
       this.setActivation(EdgeState.make(type));
       try {
	   // Each title must have a unique id to identify itself within the DB
	   this.setId( generateUniqueId() );
       }
       catch( ServiceUnavailableException se ) {
	   // The unique id could not be set therefore terminate the transaction
	   // by throwing a system exception
	   throw new EJBException( se.getMessage() );
       }

       // This is only possible in CMPs. Otherwise return a valid PK.
       return null;
   }
   
   public void setEntityContext( EntityContext lContext )
   {
      mContext = lContext;
   }
   
   public void unsetEntityContext()
   {
      mContext = null;
   }
   
   public void ejbActivate()
   {
   }
   
   public void ejbPassivate()
   {
   }
   
   public void ejbLoad()
   {
   }
   
   public void ejbStore()
   {
   }
   
   public void ejbRemove()
      throws
         RemoveException
   {
   }


   /**
   * Retrive a unique creation id to use for this bean.  This will end up
   * demarcating this bean from others when it is stored as a record
   * in the database.
   *
   * @return Returns an integer that can be used as a unique creation id.
   *
   * @throws ServiceUnavailableException Indicating that it was not possible
   *                                     to retrieve a new unqiue ID because
   *                                     the service is not available
   **/
   private int generateUniqueId()
      throws ServiceUnavailableException
   {
      int lUniqueId = -1;
      try {
         Context lContext = new InitialContext();
         
         String lSequenceName = (String) lContext.lookup( 
            "java:comp/env/SequenceName" 
         );
         SequenceGeneratorHome lHome = (SequenceGeneratorHome) PortableRemoteObject.narrow(
            lContext.lookup(
               "java:comp/env/ejb/hero/SequenceGenerator"
            ),
            SequenceGeneratorHome.class
         );
         SequenceGenerator lBean = (SequenceGenerator) lHome.create();
         lUniqueId = lBean.getNextNumber( lSequenceName );
         lBean.remove();  
      }
      catch ( NamingException ne ) {
         throw new ServiceUnavailableException( "Naming lookup failure: " + ne.getMessage() );
      }
      catch ( CreateException ce ) {
         throw new ServiceUnavailableException( "Failure while creating a generator session bean: " + ce.getMessage() );
      }
      catch ( RemoveException re ) {
         // When the Bean cannot be removed after a while it will be taken back by the container
         // therefore ignore this exception
      }
      catch ( RemoteException rte ) {
         throw new ServiceUnavailableException( "Remote exception occured while accessing generator session bean: " +  rte.getMessage() );
      }
      
      return lUniqueId;
   }
   
}
