package hero.session;
/*
*
* ProjectSoapBasicBean.java - 
* Copyright (C) 2003 Ecoo Team
* valdes@loria.fr
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import hero.entity.EdgeState;
import hero.entity.NodeState;
import hero.interfaces.BnAgentEdgeLocal;
import hero.interfaces.BnAgentEdgeLocalHome;
import hero.interfaces.BnAgentLocal;
import hero.interfaces.BnAgentLocalHome;
import hero.interfaces.BnAgentValue;
import hero.interfaces.BnEdgeLocal;
import hero.interfaces.BnEdgeLocalHome;
import hero.interfaces.BnEdgeUtil;
import hero.interfaces.BnEdgeValue;
import hero.interfaces.BnNodeHookLocal;
import hero.interfaces.BnNodeHookLocalHome;
import hero.interfaces.BnNodeHookUtil;
import hero.interfaces.BnNodeInterHookLocal;
import hero.interfaces.BnNodeInterHookValue;
import hero.interfaces.BnNodeInterHookLocalHome;
import hero.interfaces.BnNodeInterHookUtil;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodeLocalHome;
import hero.interfaces.BnNodePropertyLocal;
import hero.interfaces.BnNodePropertyValue;
import hero.interfaces.BnNodePropertyLocalHome;
import hero.interfaces.BnNodePropertyUtil;
import hero.interfaces.BnNodeUtil;
import hero.interfaces.BnNodeValue;
import hero.interfaces.BnProjectHookLocal;
import hero.interfaces.BnProjectHookLocalHome;
import hero.interfaces.BnProjectHookUtil;
import hero.interfaces.BnProjectInterHookLocal;
import hero.interfaces.BnProjectInterHookLocalHome;
import hero.interfaces.BnProjectInterHookUtil;
import hero.interfaces.BnProjectInterHookValue;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnProjectPropertyLocal;
import hero.interfaces.BnProjectPropertyLocalHome;
import hero.interfaces.BnProjectPropertyUtil;
import hero.interfaces.BnProjectPropertyValue;
import hero.interfaces.BnProjectValue;
import hero.interfaces.BnRoleLocal;
import hero.interfaces.BnRoleLocalHome;
import hero.interfaces.BnRoleUtil;
import hero.interfaces.BnRoleValue;
import hero.interfaces.BnUserLightValue;
import hero.interfaces.BnUserLocal;
import hero.interfaces.BnUserLocalHome;
import hero.interfaces.BnUserUtil;
import hero.interfaces.InvalidValueException;
import hero.interfaces.Constants;
import hero.util.DeleteEdgeException;
import hero.util.EventConstants;
import hero.util.HeroException;
import hero.util.NodeCancelledException;
import hero.util.NodeExecutingException;
import hero.util.NodeTerminatedException;
import hero.util.TimerFilter;
import hero.interfaces.JMSServicesSessionUtil;
import hero.interfaces.JMSServicesSessionLocalHome;
import hero.interfaces.JMSServicesSessionLocal;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.ejb.EJBException;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.NotificationFilter;
import javax.management.ObjectName;
import javax.naming.Context;

import java.util.Vector;

import org.apache.log4j.Category;

/**
 * Session Bean Template
 *
 * @ejb:bean name="ProjectSoapBasic"
 *           display-name="ProjectSoapBasic Bean"
 *           type="Stateful"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/ProjectSoapBasic"
 *
 * @ejb:ejb-ref	ejb-name="ProjectSoapBasic"
 *              	view-type="remote"
 *              	ref-name="ProjectSoapBasic"
 *
 * @ejb:ejb-ref ejb-name="BnProject"
 *              ref-name="myhero/BnProject"
 *
 * @ejb:transaction type="Supports"
 * @ejb.permission role-name="Admin,user"
 * @jonas.bean
 *      ejb-name="ProjectSoapBasic"
 *      jndi-name="ejb/hero/ProjectSoapBasic"
 *
 **/

public class ProjectSoapBasicBean implements SessionBean, EventConstants  {

   // -------------------------------------------------------------------------
   // Static
   // -------------------------------------------------------------------------
    // Utility variable
    private static final Category log = Category.getInstance(ProjectSessionBean.class);
   
   // -------------------------------------------------------------------------
   // Members 
   // -------------------------------------------------------------------------
   
   private SessionContext mContext;
   private BnProjectLocalHome pHome;
   private BnProjectLocal mProject;
   private JMSServicesSessionLocal jms;
   private Collection subProjects = new ArrayList();
   private ArrayList visited = new ArrayList();   
   
//	-------------------------------------------------------------------------
	 // Add and Set Methods
	 // -------------------------------------------------------------------------

	 /**
	  * Add a user to this project (This user must exist at bonita database)
	  *
	  * @param userName The name of the user to associate to the project
	  *
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void addUser(String userName) throws HeroException {
		 log.debug("addUser: name=" + userName);
		 BnUserLocalHome uHome;
		 BnRoleLocalHome rHome;
		 BnUserLocal mUser;
		 try {
			 uHome = hero.interfaces.BnUserUtil.getLocalHome();
			 rHome = hero.interfaces.BnRoleUtil.getLocalHome();
		 } catch (javax.naming.NamingException be) {
			 throw new HeroException(be.getMessage());
		 }
		 try {
			 mUser = uHome.findByName(userName);
		 } catch (javax.ejb.FinderException fe) {
			 throw new HeroException("BnUser does not exist");
		 }

		 Collection cUsers = mProject.getBnUsers();
		 cUsers.add(mUser);
		 this.addRole(userName, "BnUser BnRole");
		 this.setUserRole(userName, userName);
		 jms.sendProjectEvent(ADDUSERPROJECT,this.getName(),userName);

	 }

	 /**
	  * Add a node to the project. This method creates a node with the corresponding
	  * node type and assign to it the hero.interfaces.Constants.InitialRole role.<br><br>
	  * 
	  * Possible node types are:<br>
	  * <br>
	  * 	hero.interfaces.Constants.Nd.AND_JOIN_NODE // Manual and join node <br>
	  *	hero.interfaces.Constants.Nd.OR_JOIN_NODE // Manual or join node <br>
	  *	hero.interfaces.Constants.Nd.AND_JOIN_AUTOMATIC_NODE // Automatic and join node <br>
	  *	hero.interfaces.Constants.Nd.OR_JOIN_AUTOMATIC_NODE // Automatic or join node <br>
	  *
	  * @param name The name of the node to add to the project
	  * @param nodeType The type of the node to add to the project 
	  *
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/

	 public void addNode(String name, int nodeType) throws HeroException {
		 log.debug("addNode: name=" + name + " nodeType:");
		 BnNodeLocalHome nHome;
		 BnNodeLocal mNode;

		 try {
			 nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		 } catch (javax.naming.NamingException be) {
			 throw new HeroException(be.getMessage());
		 }		

		 try {
			 BnNodeLocal nf = nHome.findByName(name, mProject.getId());
			 throw new HeroException("Node "+ name+ " already exist in project "+ mProject.getName());
		 } catch (javax.ejb.FinderException nn) {
			 try {
				 BnNodeLocal nd = nHome.create(name, nodeType);
				 BnNodeValue ndata = nd.getBnNodeValue();
				 ndata.setCreationDate(new java.util.Date());
				 nd.setBnNodeValue(ndata);
				 Collection cNodes = mProject.getBnNodes();
				 cNodes.add(nd);
				 String creator = mContext.getCallerPrincipal().getName();
				 BnUserLocalHome uHome =	hero.interfaces.BnUserUtil.getLocalHome();
				 BnUserLocal user = uHome.findByName(creator);
				 nd.setCreator(user);
				 nd.setExecutor(user); // Only for JOnAS consistency
				 BnRoleLocalHome rHome = BnRoleUtil.getLocalHome();
				 BnRoleLocal role = rHome.findByName(hero.interfaces.Constants.INITIALROLE,mProject.getId());
				 nd.setBnRole(role);

				 if (nodeType == Constants.Nd.SUB_PROCESS_NODE)
				 {
					 this.addNodeInterHook(name,"subProcess_Start",Constants.Nd.BEFORESTART,hero.hook.Hook.BSINTERACTIVE,Constants.SUBPROCESS_START);					
					 this.addNodeInterHook(name,"subProcess_Terminate",Constants.Nd.BEFORETERMINATE,hero.hook.Hook.BSINTERACTIVE,Constants.SUBPROCESS_TERMINATE);
					 try {
						 BnProjectLocal pl = pHome.findByName(name);
						 pl.setParent(mProject.getName());
					 } catch (javax.ejb.FinderException pe) {throw new HeroException("Project "+ name+ " does not exist");}
				 }

				 // Tell the world
				 jms.sendNodeEvent(ADDNODE,this.getName(),name,nodeType,ndata.getState(),mContext.getCallerPrincipal().getName());
			 } catch (InvalidValueException ie) {
				 throw new HeroException(ie.getMessage());
			 } catch (CreateException ce) {
				 throw new HeroException(ce.getMessage());
			 } catch (FinderException fe) {
				 throw new HeroException(fe.getMessage());
			 } catch (javax.naming.NamingException ne) {
				 throw new HeroException(ne.getMessage());
			 }
		 }
	 }
	
	 /**
	  * Add a subProcess node to the project. This method creates the subProject from an existing
	  * project and creates the node associated to it. The type of created node is 
	  * hero.interfaces.Constants.Nd.SUB_PROCESS_NODE
	  *
	  * @param name The name of the node to add to the project
	  * @param projectName the name of the subProcess
	  *
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/

	 public void addNodeSubProcess(String name, String projectName) throws HeroException {
		 try{
		   String aP = mProject.getName();
		   this.initProject(projectName,name);
		   Collection nodes = mProject.getBnNodes();
		   Iterator nds = nodes.iterator();
		   while (nds.hasNext()) // first activity will be initial
		   {
			 BnNodeLocal subNode = (BnNodeLocal)nds.next();
			 if (subNode.getState()==Constants.Nd.READY)
				 this.setNodeState(subNode,Constants.Nd.INITIAL);
		   }
		   this.initProject(aP);
		   this.addNode(name,Constants.Nd.SUB_PROCESS_NODE);
		 }catch (Exception ans){throw new HeroException("Error in addNodeSubProcess");}
	 }

	 /**
	  * Add an Agent to the Project
	  *
	  * @param name The name of the Agent to add to the project
	  *
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void addAgent(String name) throws HeroException {
		 log.debug("addNode: name=");
		 BnAgentLocalHome nHome;
		 BnAgentLocal mAgent;
		 try {
			 nHome = hero.interfaces.BnAgentUtil.getLocalHome();
		 } catch (javax.naming.NamingException be) {
			 throw new HeroException(be.getMessage());
		 }

		 try {
			 BnAgentLocal nf = nHome.findByName(name, mProject.getId());
			 throw new HeroException("BnAgent "+ name+ " already exist in project "+ mProject.getName());
		 } catch (javax.ejb.FinderException nn) {

			 try {
				 BnAgentLocal ag = nHome.create(name);
				 BnAgentValue adata = ag.getBnAgentValue();
				 adata.setCreationDate(new java.util.Date());
				 ag.setBnAgentValue(adata);
				 Collection cAgents = mProject.getBnAgents();
				 cAgents.add(ag);
				 String creator = mContext.getCallerPrincipal().getName();
				 BnUserLocalHome uHome =	hero.interfaces.BnUserUtil.getLocalHome();
				 BnUserLocal user = uHome.findByName(creator);
				 ag.setCreator(user);

				 // Tell the world
				 //		jms.sendNodeEvent(ADDNODE,this.getName(),name,nodeType,ndata.getState(),mContext.getCallerPrincipal().getName());
			 } catch (InvalidValueException ie) {
				 throw new HeroException(ie.getMessage());
			 } catch (CreateException ce) {
				 throw new HeroException(ce.getMessage());
			 } catch (FinderException fe) {
				 throw new HeroException(fe.getMessage());
			 } catch (javax.naming.NamingException ne) {
				 throw new HeroException(ne.getMessage());
			 }
		 }
	 }

	 /**
	  * Add an edge to the project. Creates a transition between two nodes by specifying
	  * the name of this edge. <strong> No more implemented </strong>
	  *
	  * @param name the name ot the edge
	  *
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void addEdge(String name, String in, String out)
		 throws HeroException {
		 log.debug("addEdge: name=" + name + " in=" + in + " out=" + out);
		 throw new HeroException("no more implemented");
	 }

	 /**
	  * Add an edge between two nodes of the project [in] ---- [out]. This operation refresh
	  * the out node state. If new edge forms a cycle an exception is throwed.
	  *
	  * @param in the name of the in Node
	  * @param out the name of the out Node
	  *
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * 
	  * @throws HeroException
	  *
	  **/
	 public String addEdge(String in, String out) throws HeroException {
		 log.debug("addEdge: in=" + in + " out:" + out);
		 BnEdgeLocalHome eHome;
		 BnNodeLocalHome nHome;
		 BnNodeLocal mInNode;
		 BnNodeLocal mOutNode;
		 BnEdgeLocal mEdge;
		 try {
			 nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		 } catch (javax.naming.NamingException be) {
			 throw new HeroException(be.getMessage());
		 }
		 try {
			 mInNode = nHome.findByName(in, mProject.getId());
		 } catch (FinderException fe) {
			 throw new HeroException("In Node does not exist");
		 }
		 try {
			 mOutNode = nHome.findByName(out, mProject.getId());
		 } catch (FinderException fe) {
			 throw new HeroException("Out Node does not exist");
		 }
		 try {
			 eHome = hero.interfaces.BnEdgeUtil.getLocalHome();
		 } catch (javax.naming.NamingException be) {
			 throw new HeroException(be.getMessage());
		 }
		 visited.clear();
		 if (!checkStructuralConstraints(in, out))
			 throw new hero.util.EdgeCycleException("This edge add a cycle");
		 
		 if (checkDynamics(out)) { // check state of Nodes
			 if (checkDuplicateEdge(in, out)) {
				 try {
					 mEdge = eHome.create();
					 mEdge.setBnProject(mProject);
					 mEdge.setInBnNode(mInNode);
					 mEdge.setOutBnNode(mOutNode);

					 jms.sendEdgeEvent(ADDEDGE,this.getName(),mEdge.getName(),mInNode.getName(),mOutNode.getName(),mContext.getCallerPrincipal().getName());

				 } catch (InvalidValueException ie) {
					 throw new HeroException(ie.getMessage());
				 } catch (CreateException ce) {
					 throw new HeroException(ce.getMessage());
				 }
			 } else {
				 throw new hero.util.DuplicatedEdgeException(
					 "Cannot add edge. BnEdge between nodes already exist");
			 }
		 } else {
			 throw new hero.util.NodeStartedException(
				 "Cannot add edge. " + out + " is already started");
		 }
		 /*visited.clear();
		 if (!checkStructuralConstraints(in, out))
			 throw new hero.util.EdgeCycleException("This edge add a cycle");*/
		
		 // update the current graph state
		 int edgeState = hero.interfaces.Constants.edgeTransition[hero.interfaces.Constants.Ed.CONDITION_TRUE][mInNode.getState()];
		 this.setEdgeState(mEdge, edgeState);
		 //	mEdge.setState(edgeState);
		 int newOperation = mOutNode.getActivation().state(mOutNode);
		 int newState = mOutNode.getTransition().computeState(mOutNode, newOperation);
		 if (newState == hero.interfaces.Constants.Nd.BAD_TRANSITION) {
			 throw new HeroException("AddEdge generated a bad Node transition");
		 }
		 this.setNodeState(mOutNode, newState);
		 //	jms.sendNodeEvent(READYINITIAL,this.getName(),nOut.getName(),nOut.getType(),INITIAL,context.getCallerPrincipal().getName());

		 return mEdge.getName();
	 }

	 /**
	  * Add an AgentEdge to the Project [in] ---- [out]
	  *
	  * @param in the name of the in Agent
	  * @param out the name of the out Node
	  *
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * 
	  * @throws HeroException
	  *
	  **/
	 public String addAgentEdge(String in, String out) throws HeroException {
		 log.debug("addAgentEdge: in=" + in + " out:" + out);
		 BnAgentEdgeLocalHome eHome;
		 BnNodeLocalHome nHome;
		 BnAgentLocalHome aHome;
		 BnAgentLocal mInAgent;
		 BnNodeLocal mOutNode;
		 BnAgentEdgeLocal mEdge;
		 try {
			 aHome = hero.interfaces.BnAgentUtil.getLocalHome();
		 } catch (javax.naming.NamingException be) {
			 throw new HeroException(be.getMessage());
		 }
		 try {
			 mInAgent = aHome.findByName(in, mProject.getId());
		 } catch (FinderException fe) {
			 throw new HeroException("In Agent does not exist");
		 }
		 try {
			 nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		 } catch (javax.naming.NamingException be) {
			 throw new HeroException(be.getMessage());
		 }
		 try {
			 mOutNode = nHome.findByName(out, mProject.getId());
		 } catch (FinderException fe) {
			 throw new HeroException("Out Node does not exist");
		 }
		 try {
			 eHome = hero.interfaces.BnAgentEdgeUtil.getLocalHome();
		 } catch (javax.naming.NamingException be) {
			 throw new HeroException(be.getMessage());
		 }
		 if (checkDynamics(out)) { // check state of Nodes
			 if (checkDuplicateAgentEdge(in, out)) {
				 try {
					 mEdge = eHome.create();
					 mEdge.setBnProject(mProject);
					 mEdge.setInBnAgent(mInAgent);
					 mEdge.setOutBnNode(mOutNode);

					 //jms.sendEdgeEvent(ADDEDGE,this.getName(),mEdge.getName(),mInNode.getName(),mOutNode.getName(),mContext.getCallerPrincipal().getName());

				 } catch (InvalidValueException ie) {
					 throw new HeroException(ie.getMessage());
				 } catch (CreateException ce) {
					 throw new HeroException(ce.getMessage());
				 }
			 } else {
				 throw new hero.util.DuplicatedEdgeException("Cannot add agentEdge. AgentEdge between nodes already exist");
			 }
		 } else {
			 throw new hero.util.NodeStartedException(
				 "Cannot add agentEdge. " + out + " is already started");
		 }

		 // update the current graph state
		 this.setAgentEdgeState(mEdge, hero.interfaces.Constants.Ed.INITIAL);

		 int newOperation = mOutNode.getActivation().state(mOutNode);
		 int newState = mOutNode.getTransition().computeState(mOutNode, newOperation);
		 if (newState == hero.interfaces.Constants.Nd.BAD_TRANSITION) {
			 throw new HeroException("AddEdge generated a bad Node transition");
		 }
		 this.setNodeState(mOutNode, newState);
		 //	jms.sendNodeEvent(READYINITIAL,this.getName(),nOut.getName(),nOut.getType(),INITIAL,context.getCallerPrincipal().getName());

		 return mEdge.getName();
	 }

	 /**
	  * Add a role to the project. Creates a role within this project. The role is specific of this project
	  *
	  * @param name the name of the role
	  * @param description the description of the role
	  *
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * 
	  * @throws HeroException
	  *
	  **/
	 public void addRole(String roleName, String description)
		 throws HeroException {
		 BnRoleLocalHome roleh;
		 log.debug("addRole: roleName=" + roleName + " description=" + description);
		 try {
			 roleh = BnRoleUtil.getLocalHome();
		 } catch (javax.naming.NamingException ne) {
			 throw new HeroException(ne.getMessage());
		 }

		 try {
			 BnRoleLocal nf = roleh.findByName(roleName, mProject.getId());
			 throw new HeroException("BnRole "+ roleName+ " already exist in project "+ mProject.getName());
		 } catch (javax.ejb.FinderException nn) {

			 BnRoleValue rv = new BnRoleValue();
			 rv.setName(roleName);
			 rv.setDescription(description);
			 try {
				 BnRoleLocal rl = roleh.create(rv);
				 mProject.getBnRoles().add(rl);
				 jms.sendRoleEvent(ADDROLE,this.getName(),roleName);
			 } catch (javax.ejb.CreateException ce) {
				 throw new HeroException(ce.getMessage());
			 }
		 }
	 }
	
	 /**
	  * Add a new iteration between two nodes. This methods sets an iteration which is stopped when
	  * the lastNode property "iterate" is false.  
	  *
	  * @param from the name of the first node
	  * @param to the name of the last node
	  *
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * 
	  * @throws HeroException
	  *
	  **/
	 public void addIteration(String from, String to) throws HeroException {
		 log.debug("addIteration: from=" + from + " to=" + to);
		 if (!this.checkStructuralConstraints(from,to))
		 {
		   this.setNodeProperty(from,"iterate","true",false);
		   this.setProperty("iterate_"+from+"-"+to,from);
		 }
		 else
			 throw new HeroException("This iteration is not possible...");
	 }

	 /**
	  * Add hook to project. Add an existing hook file to the project. This type of hook use a 
	  * Java or TCL file loaded at run time. <strong> This functionality is disabled. Use {@link #addInterHook(String,String,int,String) addInterHook}
	  * method to add a beanshell hook to the project. </strong>
	  *
	  * @param hookName the name of the hook
	  * @param eventName the event
	  * @param hookType the type
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void addHook(String hookName, String eventName, int hookType)
		 throws HeroException {
		 log.debug("addHook: hookName=" + hookName + " eventName=" + eventName);
		 BnProjectHookLocalHome pHome;
		 BnProjectHookLocal ppLocal;

		 try {
			 pHome = BnProjectHookUtil.getLocalHome();
		 } catch (javax.naming.NamingException ne) {
			 throw new HeroException(ne.getMessage());
		 }

		 try {
			 BnProjectHookLocal pf = pHome.findByName(mProject.getId(), hookName);
			 throw new HeroException("BnProject hook "+ hookName	+ " already exist in project "+ mProject.getName());
		 } catch (javax.ejb.FinderException pn) {

			 try {
				 BnProjectHookLocal h =	pHome.create(hookName, eventName, hookType);
				 h.setBnProject(mProject);
			 } catch (CreateException fe) {
				 throw new HeroException(fe.getMessage());
			 } catch (InvalidValueException ie) {
				 throw new HeroException(ie.getMessage());
			 }
		 }
	 }

	 /**
	  * Add hook to a node. Add an existing hook file to the node. This type of hook use a 
	  * Java or TCL file loaded at run time. <strong> This functionality is disabled.
	  * Use {@link #addNodeInterHook(String,String,String,int,String) addNodeInterHook}
	  * method to add a beanshell hook to the node. </strong>
	  *
	  * @param nodeName the name of the node
	  * @param hookName the name of the hook
	  * @param eventName the name of the event
	  * @param hookType the type of the hook
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void addNodeHook(String nodeName,String hookName,String eventName,int hookType)	throws HeroException {
		 log.debug("addNodeHook: node="+ nodeName+ " hookName="+ hookName+ " eventName="	+ eventName);
		 BnNodeHookLocalHome pHome;
		 BnNodeHookLocal ppLocal;

		 BnNodeLocal nd = mProject.getBnNode(nodeName);
		 try {
			 pHome = BnNodeHookUtil.getLocalHome();
		 } catch (javax.naming.NamingException ne) {
			 throw new HeroException(ne.getMessage());
		 }

		 try {
			 BnNodeHookLocal nf = pHome.findByName(nd.getId(), hookName);
			 throw new HeroException("BnNode hook "+ hookName+ " already exist in node "+ nodeName);
		 } catch (javax.ejb.FinderException nn) {

			 try {
				 BnNodeHookLocal h = pHome.create(hookName, eventName, hookType);
				 h.setBnNode(nd);
			 } catch (CreateException fe) {
				 throw new HeroException(fe.getMessage());
			 } catch (InvalidValueException ie) {
				 throw new HeroException(ie.getMessage());
			 }
		 }
	 }
	
	 /**
	  * Add interhook to a node. Set a new hook to this node by setting the eventName, hookType and
	  * script values.<br><br>
	  * 
	  * The possible values of the argumets are:<br><br>
	  * <strong>eventName:<br></strong>
	  * 	hero.interfaces.Constants.Nd.BEFORESTART<br>
	  *  hero.interfaces.Constants.Nd.AFTERSTART<br>
	  *	hero.interfaces.Constants.Nd.BEFORETERMINATE<br>
	  *	hero.interfaces.Constants.Nd.AFTERTERMINATE<br>
	  *	hero.interfaces.Constants.Nd.ONCANCEL<br>
	  *	hero.interfaces.Constants.Nd.ANTICIPATE<br><br>
	  *
	  * <strong>hookType:<br></strong>
	  * 
	  *  hero.hook.Hook.BSINTERACTIVE<br><br>
	  * 
	  * <strong>script:<br></strong>
	  * The beanshell source code associated to this hook.<br><br>
	  * 
	  * <strong>The following sample code illustrates how to use this method:<br><br></strong>
	  *
	  *  ...<br>
	  *  String script = "import hero.interfaces.*;\n"<br>
	  * 		            + "import hero.interfaces.BnNodeLocal;\n"<br>
	  *			        + "beforeStart (Object b,Object n) {\n\n\n"<br>
	  *					+ "System.out.println(\"Hook Test\");\n"<br>
	  *			        + "}";<br>
	  *  <strong>yourProjectSessionInstance</strong>.addNodeInterHook("NodeName","Hook_Name",Constants.Nd.BEFORESTART,hero.hook.Hook.BSINTERACTIVE,script);
	  *
	  * @param nodeName the name of the node
	  * @param hookName the name of the hook
	  * @param eventName the name of the event
	  * @param hookType the type of the hook
	  * @param script the value of the script 
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void addNodeInterHook(String nodeName,String hookName,String eventName,int hookType,String value)
		 throws HeroException {
		 log.debug("addNodeHook: node="+ nodeName+ " hookName="+ hookName+ " eventName="+ eventName+ " value="+ value);
		 BnNodeInterHookLocalHome pHome;
		 BnNodeInterHookLocal ppLocal;

		 BnNodeLocal nd = mProject.getBnNode(nodeName);
		 try {
			 pHome = BnNodeInterHookUtil.getLocalHome();
		 } catch (javax.naming.NamingException ne) {
			 throw new HeroException(ne.getMessage());
		 }

		 try {
			 BnNodeInterHookLocal nf = pHome.findByName(nd.getId(), hookName);
			 throw new HeroException("Interactive node hook "+ hookName+ " already exist in node "+ nodeName);
		 } catch (javax.ejb.FinderException nn) {
			 try {
				 BnNodeInterHookLocal h = pHome.create(hookName, eventName, hookType, value);
				 h.setBnNode(nd);
			 } catch (CreateException fe) {
				 throw new HeroException(fe.getMessage());
			 } catch (InvalidValueException ie) {
				 throw new HeroException(ie.getMessage());
			 }
		 }
	 }

	 /**
	  * Add interhook to the project. Creates new hook associated to all project activites.
	  * See the {@link #addNodeInterHook(String,String,String,int,String) addNodeInterHook} method in order
	  * to identify arguments values.
	  *
	  * @param hookName the name of the hook
	  * @param eventName the name of the event
	  * @param hookType the type of the hook
	  * @param script the value of the script 
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void addInterHook(String hookName,String eventName,int hookType,String value)
		 throws HeroException {
		 log.debug("addNodeHook: hookName="+ hookName+ " eventName="+ eventName+ " value="+ value);
		 BnProjectInterHookLocalHome pHome;
		 BnProjectInterHookLocal ppLocal;

		 try {
			 pHome = BnProjectInterHookUtil.getLocalHome();
		 } catch (javax.naming.NamingException ne) {
			 throw new HeroException(ne.getMessage());
		 }

		 try {
			 BnProjectInterHookLocal nf = pHome.findByName(mProject.getId(), hookName);
			 throw new HeroException("Interactive hook "+ hookName+ " already exist in project "+ mProject.getName());
		 } catch (javax.ejb.FinderException nn) {
			 try {
				 BnProjectInterHookLocal h = pHome.create(hookName, eventName, hookType, value);
				 h.setBnProject(mProject);
			 } catch (CreateException fe) {
				 throw new HeroException(fe.getMessage());
			 } catch (InvalidValueException ie) {
				 throw new HeroException(ie.getMessage());
			 }
		 }
	 }

	 // -------------------------------------------------------------------------
	 // Delete business methods
	 // -------------------------------------------------------------------------

	 /**
	  * Delete a node from the project. If this node is in executing, terminated or cancelled state, the
	  * method throws an execption
	  *
	  * @param name the name ot the edge
	  *
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void deleteNode(String name) throws HeroException {
		 // check deletion conditions
		 log.debug("deleteNode: name=" + name);
		 
		 BnNodeLocal nd = mProject.getBnNode(name);
		 if (nd.isExecuting()) {
			 throw new NodeExecutingException("Cannot delete node. It is executing");
		 }
		 if (nd.isTerminated()) {
			 throw new NodeTerminatedException("Cannot delete node. It is terminated");
		 }
		 if (nd.isCancelled()) {
			 throw new NodeCancelledException("Cannot delete node. It is cancelled");
		 }
		
		 if (this.getNodeType(name)==Constants.Nd.SUB_PROCESS_NODE)
		 {
			try {
			   BnProjectLocal plocal = pHome.findByName(name);
			   Collection nodes = plocal.getBnNodes();
			   Iterator nds = nodes.iterator();
			   while (nds.hasNext())            // Verifies if the project has any subProcesses
			   {
				  BnNodeLocal nod = (BnNodeLocal)nds.next();
				  if (nod.getType()==Constants.Nd.SUB_PROCESS_NODE)
					throw new HeroException("Error in delete project: You have to delete subProcesses first !!");
			   }
			   plocal.remove();
			   jms.sendProjectEvent(DELETEPROJECT, name);
			 } catch (RemoveException pre) {
			   throw new HeroException("Cannot remove project " + pre.getMessage());
			 } catch (FinderException pfe) {
			   throw new HeroException("Cannot find project " + pfe.getMessage());
			 }
		 }
		
		 Collection edgesIn = nd.getInBnEdges();
		 Object[] edai = edgesIn.toArray();
		 for (int i = 0; i < edai.length; i++) {
			 edgeDeletion((BnEdgeLocal) edai[i]);
		 }
		 Collection edgesOut = nd.getOutBnEdges();
		 Object[] edao = edgesOut.toArray();
		 for (int i = 0; i < edao.length; i++) {
			 edgeDeletion((BnEdgeLocal) edao[i]);
		 }
		 try {
			 int state = nd.getState();
			 int type = nd.getType();
			 nd.remove();
			 jms.sendNodeEvent(DELETENODE,this.getName(),name,type,state,mContext.getCallerPrincipal().getName());
			 removeOldNotification(name + ":" + this.getName());
		 } catch (RemoveException re) {
			 throw new HeroException("Cannot remove node " + re.getMessage());
		 }
		 log.debug("deleteNode -end: name=" + name);
	 }

	 /**
	  * Delete an edge to the project. If out node of this edge is in executing, terminated or cancelled state, the
	  * method throws an execption.
	  *
	  * @param name the name ot the edge
	  *
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void deleteEdge(String name) throws HeroException {
		 // check deletion conditions
		 log.debug("deleteEdge: name=" + name);
		 BnEdgeLocalHome eh;
		 BnEdgeLocal ed;
		 try {
			 eh = BnEdgeUtil.getLocalHome();
		 } catch (javax.naming.NamingException ne) {
			 throw new HeroException(ne.getMessage());
		 }
		 try {
			 ed = eh.findByName(name, mProject.getId());
		 } catch (FinderException fe) {
			 throw new HeroException("edge does not exists " + name);
		 }
		 String nodeIn = ed.getInBnNode().getName();
		 String nodeOut = ed.getOutBnNode().getName();
		 edgeDeletion(ed);
		 jms.sendEdgeEvent(DELETEEDGE,this.getName(),name,nodeIn,nodeOut,mContext.getCallerPrincipal().getName());
		 log.debug("deleteEdge -end: name=" + name);
	 }

	 /** Actually deletes an edge and restore the state of the outgoing node
	  *  <strong>Not a business method</strong>
	 **/

	 public void edgeDeletion(BnEdgeLocal ed) throws HeroException {
		 BnNodeLocal nd = ed.getOutBnNode();
		 if (nd.isExecuting()) {
			 throw new DeleteEdgeException("Cannot delete edge. Out node executing");
		 }
		 if (nd.isTerminated()) {
			 throw new DeleteEdgeException("Cannot delete edge. Out node terminated");
		 }
		 if (nd.isCancelled()) {
			 throw new DeleteEdgeException("Cannot delete edge. Out node cancelled");
		 }
		 try {
			 ed.remove();
			 int newOperation = nd.getActivation().state(nd);
			 int newState = nd.getTransition().computeState(nd, newOperation);
			 if (newState == hero.interfaces.Constants.Nd.BAD_TRANSITION) {
				 throw new HeroException("AddEdge generated a bad BnNode transition");
			 }
			 this.setNodeState(nd, newState);
		 } catch (RemoveException e) {
			 throw new HeroException("deleteNode failed " + e.getMessage());
		 }
	 }


	 /**
	  * Detete a hook.<strong> This functionality is disabled.
	  * Use {@link #deleteInterHook(String) deleteInterHook} 
	  * method to delete a hook of this project. </strong>
	  *
	  * @param hookName the name of the hook
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void deleteHook(String hookName) throws HeroException {
		 BnProjectHookLocalHome pHome;
		 BnProjectHookLocal ppLocal;

		 try {
			 pHome = BnProjectHookUtil.getLocalHome();
		 } catch (javax.naming.NamingException ne) {
			 throw new HeroException(ne.getMessage());
		 }

		 try {
			 BnProjectHookLocal ph =	pHome.findByName(mProject.getId(), hookName);
			 ph.remove();
		 } catch (FinderException fe) {
			 throw new HeroException(fe.getMessage());
		 } catch (RemoveException rm) {
			 throw new HeroException(rm.getMessage());
		 }
	 }

	 /**
	  * Delete a node hook.<strong> This functionality is disabled.
	  * Use {@link #deleteNodeInterHook(String,String) deleteNodeInterHook}
	  * method to delete a hook of this node. </strong>
	  *
	  * @param nodeName the name of the node
	  * @param hookName the name 
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void deleteNodeHook(String nodeName, String hookName)
		 throws HeroException {
		 BnNodeHookLocalHome pHome;
		 BnNodeHookLocal ppLocal;
		 BnNodeLocal nd = mProject.getBnNode(nodeName);
		 try {
			 pHome = BnNodeHookUtil.getLocalHome();
		 } catch (javax.naming.NamingException ne) {
			 throw new HeroException(ne.getMessage());
		 }
		 try {
			 BnNodeHookLocal nh = pHome.findByName(nd.getId(), hookName);
			 nh.remove();
		 } catch (FinderException fe) {
			 throw new HeroException(fe.getMessage());
		 } catch (RemoveException rm) {
			 throw new HeroException(rm.getMessage());
		 }
	 }

	 /**
	   * Delete a node interHook.
	   *
	   * @param nodeName the name of the node
	   * @param hookName the name of the interHook
	   * @ejb:interface-method view-type="both"
	   * @ejb:transaction type="Required"
	   * @throws HeroException
	   *
	   **/
	 public void deleteNodeInterHook(String nodeName, String interHookName)
		 throws HeroException {
		 BnNodeInterHookLocalHome pHome;
		 BnNodeInterHookLocal ppLocal;
		 BnNodeLocal nd = mProject.getBnNode(nodeName);
		 try {
			 pHome = BnNodeInterHookUtil.getLocalHome();
		 } catch (javax.naming.NamingException ne) {
			 throw new HeroException(ne.getMessage());
		 }
		 try {
			 BnNodeInterHookLocal nh =
				 pHome.findByName(nd.getId(), interHookName);
			 nh.remove();
		 } catch (FinderException fe) {
			 throw new HeroException(fe.getMessage());
		 } catch (RemoveException rm) {
			 throw new HeroException(rm.getMessage());
		 }
	 }

	 /**
	   * Detete a interHook of this project. This method deletes this hook from all projects nodes.
	   *
	   * @param hookName the name of the hook
	   * @ejb:interface-method view-type="both"
	   * @ejb:transaction type="Required"
	   * @throws HeroException
	   *
	   **/
	 public void deleteInterHook(String interHookName) throws HeroException {
		 BnProjectInterHookLocalHome pHome;
		 BnProjectInterHookLocal ppLocal;

		 try {
			 pHome = BnProjectInterHookUtil.getLocalHome();
		 } catch (javax.naming.NamingException ne) {
			 throw new HeroException(ne.getMessage());
		 }

		 try {
			 BnProjectInterHookLocal ph =
				 pHome.findByName(mProject.getId(), interHookName);
			 ph.remove();
		 } catch (FinderException fe) {
			 throw new HeroException(fe.getMessage());
		 } catch (RemoveException rm) {
			 throw new HeroException(rm.getMessage());
		 }
	 }

	 /**
	  * Delete a property of the project. Deletes the project property associated to this key.
	  *
	  * @param key the key of the property
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void deleteProperty(String key) throws HeroException {
		 log.debug("deleteProperty key=" + key);

		 BnProjectPropertyLocalHome pHome;
		 BnProjectPropertyLocal ppLocal;

		 try {
			 pHome = BnProjectPropertyUtil.getLocalHome();
		 } catch (javax.naming.NamingException ne) {
			 throw new HeroException(ne.getMessage());
		 }
		 Collection c = null;
		 try {
			 c = pHome.findByTheKey(mProject.getId(), key);
		 } catch (FinderException fe) {
			 throw new HeroException(fe.getMessage());
		 }

		 if (!c.isEmpty()) {
			 try {
				 ppLocal = (BnProjectPropertyLocal) (c.toArray())[0];
				 ppLocal.remove();
			 } catch (RemoveException rm) {
				 throw new HeroException(rm.getMessage());
			 }

		 } else {
			 throw new HeroException("Property not found :" + key);
		 }
	 }

	 /**
	  * Delete a property of a node. Deletes the node property associated to this key 
	  *
	  * @param nodeName the name of the node
	  * @param key the key of the property
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void deleteNodeProperty(String nodeName, String key)
		 throws HeroException {
		 log.debug("deleteNodeProperty node=" + nodeName + " key=" + key);

		 BnNodePropertyLocalHome pHome;
		 BnNodePropertyLocal propertyLocal = null;

		 BnNodeLocal nd = mProject.getBnNode(nodeName);

		 try {
			 pHome = BnNodePropertyUtil.getLocalHome();
		 } catch (javax.naming.NamingException ne) {
			 throw new HeroException(ne.getMessage());
		 }
		 Collection c = null;
		 try {
			 c = pHome.findByTheKey(nd.getId(), key);
		 } catch (FinderException fe) {
			 throw new HeroException(fe.getMessage());
		 }

		 if (!c.isEmpty()) {
			 try {
				 propertyLocal = (BnNodePropertyLocal) (c.toArray())[0];
				 propertyLocal.remove();
			 } catch (RemoveException rm) {
				 throw new HeroException(rm.getMessage());
			 }

		 } else {
			 throw new HeroException("Property not found :" + key);
		 }
	 }

	 // -------------------------------------------------------------------------
	 // Set business methods
	 // -------------------------------------------------------------------------

	/**
	 * Set edit node changes. Only for GraphEditor client
	 *
	 * @param name of the node
	 * @param role of the node
	 * @param description of the node
	 * @param deadline of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void setEditNode(String node, String role, String description, long deadline) throws HeroException {
		log.debug("setEditNode:  node=" + node + " role=" + role +" description=" + description + " deadline=" + deadline);
		this.setNodeDeadline(node,deadline);
		this.setNodeDescription(node,description);
		this.setNodeRole(node,role);		
	}

	 /**
	  * Set a role to this user. This role is assigned to the user in the context of
	  * the actual project.
	  *
	  * @param userName the name of the user
	  * @param roleName the name of the role
	  *
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void setUserRole(String userName, String roleName)
		 throws HeroException {
		 log.debug("setUserRole: user=" + userName + " role=" + roleName);
		 BnRoleLocalHome roleh;
		 BnUserLocalHome userh;
		 try {
			 roleh = BnRoleUtil.getLocalHome();
			 userh = BnUserUtil.getLocalHome();
		 } catch (javax.naming.NamingException ne) {
			 throw new HeroException(ne.getMessage());
		 }
		 try {
			 /*NamingManagementLocal namingManagementLocal = NamingManagementUtil.getLocalHome().create();			
			 BnUserLocal user = userh.findByName(namingManagementLocal.getUsersRootName() + namingManagementLocal.getSeparator()+userName);*/
			 BnUserLocal user = userh.findByName(userName);
			 BnRoleLocal role = roleh.findByName(roleName, mProject.getId());
			 user.getBnRoles().add(role);
			 jms.sendUserRoleEvent(SETUSERROLE,this.getName(),userName,roleName);
		 } catch (Exception fe) {
			 throw new HeroException(fe.getMessage());
		 }
	 }

	 /**
	  * Propagate node properties. Propagates all node properties to connected nodes. 
	  * @param nodeName the name of the Node
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void propagateNodeProperties(String nodeName) throws HeroException {
		 log.debug("propagateNodeProperties: name=" + nodeName);

		 BnNodePropertyLocalHome pHome;
		 BnNodePropertyLocal propertyLocal = null;

		 BnNodeLocal nd = mProject.getBnNode(nodeName);

		 Collection c = nd.getBnProperties();
		 Collection c2 = nd.getOutBnEdges();
		 if ((!c.isEmpty()) && (!c2.isEmpty())) {
			 BnNodePropertyLocal p;
			 BnEdgeLocal ed;
			 for (Iterator i = c.iterator(); i.hasNext();) {
				 p = (BnNodePropertyLocal) i.next();
				 if (p.getPropagate() && !p.getTheKey().equals("iterate")) {
					 for (Iterator i2 = c2.iterator(); i2.hasNext();) {
						 ed = (BnEdgeLocal) i2.next();
						 this.setNodeProperty((ed.getOutBnNode()).getName(),p.getTheKey(),p.getTheValue());
					 }
				 }
			 }
		 }
	 }

	 /**
	  * Sets the role of an activity. Changes the role of this activity
	  *
	  * @param name the name of the activity
	  * @param role the name of the role
	  *
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void setNodeRole(String name, String role) throws HeroException {
		 log.debug("setBnNodeRole: name=" + name + " role=" + role);
		 BnRoleLocalHome rh;
		 BnNodeLocalHome nh;
		 BnNodeLocal nl;
		 BnRoleLocal rl;
		 try {
			 rh = BnRoleUtil.getLocalHome();
			 nh = BnNodeUtil.getLocalHome();
		 } catch (javax.naming.NamingException ne) {
			 throw new HeroException(ne.getMessage());
		 }
		 try {
			 nl = nh.findByName(name, mProject.getId());
		 } catch (FinderException fe) {
			 throw new HeroException("BnNode does not exists " + name);
		 }
		 try {
			 rl = rh.findByName(role, mProject.getId());
		 } catch (FinderException fe) {
			 throw new HeroException("BnRole does not exists " + role);
		 }
		 nl.setBnRole(rl);
		 jms.sendProjectEvent(CHANGEROLE, this.getName());
	 }

	 /**
	  * Set the node state. Set start date of this node when starts the execution and the end date when node
	  * is terminated.
	  *
	  * @param name name of the node
	  * @param state new state of the node
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void setNodeState(BnNodeLocal node, int state)
		 throws HeroException {
		 log.debug("setBnNodeState: name=" + node.getName() + " state=" + state);
		 node.setState(state);
		 if (state == hero.interfaces.Constants.Nd.EXECUTING) {
			 node.setStartDate(new java.util.Date());
		 }
		 if (state == hero.interfaces.Constants.Nd.TERMINATED) {
			 node.setEndDate(new java.util.Date());
			 this.propagateNodeProperties(node.getName());
		 }
		 jms.sendNodeEvent(SETNODESTATE,this.getName(),node.getName(),node.getType(),state,mContext.getCallerPrincipal().getName());

	 }

	 /**
	  * Set the edge state
	  *
	  * @param bean of the edge
	  * @param new state of the edge
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void setEdgeState(BnEdgeLocal edge, int state)
		 throws HeroException {
		 log.debug("setEdgeState: name=" + edge.getName() + " state=" + state);
		 edge.setState(state);
		 jms.sendEdgeEvent(SETEDGESTATE,this.getName(),edge.getName(),edge.getInBnNode().getName(),edge.getOutBnNode().getName(),mContext.getCallerPrincipal().getName());
	 }

	 /**
	  * Set the agent state
	  *
	  * @param agent bean of the Agent
	  * @param new state of the Agent
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void setAgentState(BnAgentLocal agent, int state)
		 throws HeroException {
		 log.debug(
			 "setBnAgentState: name=" + agent.getName() + " state=" + state);
		 agent.setState(state);
		 //	jms.sendEdgeEvent(SETEDGESTATE,this.getName(),edge.getName(),edge.getInBnNode().getName(),edge.getOutBnNode().getName(),mContext.getCallerPrincipal().getName());
	 }

	 /**
	  * Set the agentEdge state
	  *
	  * @param agentEdge bean of the BnAgentEdge
	  * @param new state of the BnAgentEdge
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void setAgentEdgeState(BnAgentEdgeLocal agentEdge, int state)
		 throws HeroException {
		 log.debug("setBnAgentEdgeState: name="+ agentEdge.getName()+ " state="+ state);
		 agentEdge.setState(state);
		 //	jms.sendEdgeEvent(SETEDGESTATE,this.getName(),edge.getName(),edge.getInBnNode().getName(),edge.getOutBnNode().getName(),mContext.getCallerPrincipal().getName());
	 }

	 /**
	  * Set the edge condition. This condition is evaluated at run-time in order to perform activity transition.
	  * To set the condition you have to use beanshell language (Lightweight Scripting for Java).<br>
	  * <strong>The next line illustrates the use of setEdgeCondition method:<br><br></strong>
	  * 
	  * ...<br>
	  * String condition = "partial_status.equals(\"ok\")";<br>  
	  * <strong>yourProjectSessionInstance</strong>.setEdgeCondition("edgeName",condition);<br><br>
	  * 
	  * The engine evaluates on the fly the condition by using the If statement. In the previous sample,
	  * if the value of the property "partial_status" is equals to "ok" the result of this condition is true.
	  *
	  * @param edge edge name
	  * @param condition condition of the edge
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void setEdgeCondition(String edge, String condition)
		 throws HeroException {
		 log.debug("setEdgeCondition: name=" + edge + " condition=" + condition);
		 BnEdgeLocal e = mProject.getBnEdge(edge);
		 e.setCondition(condition);
	 }
	
	 /**
	  * Set the node deadline. Activity deadline is the lastest date in which the activity must be finished.
	  *
	  * @param name the name of the node
	  * @param date date of the deadline
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void setNodeDeadline(String name, long date) throws HeroException {
		 log.debug("setBnNodeDeadline: name=" + name + " deadline=" + date);

		 BnNodeLocal nd = mProject.getBnNode(name);
		 nd.setDeadline(new java.sql.Date(date));
		 String projectName = this.getName();
		 jms.sendNodeEvent(SETNODEDEADLINE,projectName,name,nd.getType(),nd.getState(),mContext.getCallerPrincipal().getName());

		 try {
			 String type = name + ":" + projectName;
			 removeOldNotification(type);

			 /*				ObjectName timer = new ObjectName("jboss:service=timer");
							 ObjectName receiver = new ObjectName("jboss:service=TimerReceiver");*/
			 ObjectName timer = new ObjectName("jonas:type=service,name=Timer");
			 ObjectName receiver =
				 new ObjectName("jonas:type=service,name=TimerReceiver");
			 List list = MBeanServerFactory.findMBeanServer(null);
			 MBeanServer server = (MBeanServer) list.iterator().next();
			 Date lNext = new Date(date);
			 NotificationFilter filter = new TimerFilter();
			 server.addNotificationListener(timer, receiver, filter, null);
			 Integer id =(Integer) server.invoke(timer, "addNotification", new Object[] { type,
										 //type
										 name, //message
										 "bonita", //user object
										 lNext, //delay time
										 },
										 new String[] {
										 "".getClass().getName(),
										 "".getClass().getName(),
										 "java.lang.Object",
										 Date.class.getName()});
		 } catch (Exception e) {
			 e.printStackTrace();
			 throw new HeroException(e.getMessage());
		 }

	 }

	 /**
	  * Set the node description. Node description represents, explicity, execution related information 
	  * of this task. 
	  *
	  * @param name the name of the node
	  * @param descriptio description of the node
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void setNodeDescription(String name, String description)
		 throws HeroException {
		 log.debug("setNodeDescription: name="+ name+ " description="+ description);
		 BnNodeLocal nd = mProject.getBnNode(name);
		 nd.setDescription(description);
	 }

	 /**
	  * Set the node hook value.
	  *
	  * @param name name of the node
	  * @param hook hookName of the node
	  * @param value new script value of this hook
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void setNodeInterHookValue(String node, String hook, String value)
		 throws HeroException {
		 log.debug("setBnNodeInterHookValue: node="+ node+ " hook="+ hook+ " value="+ value);
		 BnNodeLocal nd = mProject.getBnNode(node);
		 Collection hooks = nd.getBnInterHooks();
		 for (Iterator ih = hooks.iterator(); ih.hasNext();) {
			 BnNodeInterHookLocal hk = (BnNodeInterHookLocal) ih.next();
			 if (hk.getName().equals(hook)) {
				 hk.setScript(value);
			 }
		 }
	 }

	 /**
	  * Set the hook value. This value is the new hook script associated to all project nodes 
	  *
	  * @param hook hookName of the node
	  * @param value new script value of this hook
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void setInterHookValue(String hook, String value)
		 throws HeroException {
		 log.debug("setBnNodeInterHookValue:  hook=" + hook + " value=" + value);
		 Collection hooks = mProject.getBnInterHooks();
		 for (Iterator ih = hooks.iterator(); ih.hasNext();) {
			 BnProjectInterHookLocal hk = (BnProjectInterHookLocal) ih.next();
			 if (hk.getName().equals(hook)) {
				 hk.setScript(value);
			 }
		 }
	 }

	 /**
	  * Set the node in traditional mode. When a node is traditional the anticipable attribute is false. This
	  * method must be used if you want to execute this activity in a traditional model.
	  *
	  * @param name name of the node
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void setNodeTraditional(String name) throws HeroException {
		 log.debug("setBnNodeTraditional: name=" + name);
		 BnNodeLocal nd = mProject.getBnNode(name);
		 nd.setAnticipable(false);
		 nd.setTransition(NodeState.make(nd.getType(), false));

	 }

	 /**
	  * Set the node in anticipable mode.
	  *
	  * @param name the name of the node
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void setNodeAnticipable(String name) throws HeroException {
		 log.debug("setBnNodeAnticipable: name=" + name);
		 BnNodeLocal nd = mProject.getBnNode(name);
		 nd.setAnticipable(true);
		 nd.setTransition(NodeState.make(nd.getType(), true));
	 }

	 /**
	  * Set the node in automatic  mode. The responsible of the activity execution is now the engine.
	  *
	  * @param name the name of the node
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void setNodeAutomatic(String name) throws HeroException {
		 log.debug("setBnNodeAutomatic: name=" + name);
		 BnNodeLocal nd = mProject.getBnNode(name);
		 //nd.setAutomatic(true);
		 nd.setTransition(NodeState.make(5, true));
		 //nd.setTransition(NodeState.make(nd.getType(),true));
		 //	this.addNodeHok(name,"hero.hook.AutomaticHook",hero.interfaces.Constants.Nd.CREATE,Hook.JAVA);
	 }

	 /**
	  * Set the name of the node executor. Set the name of the user which is executing the activity.
	  *
	  * @param name the name of the node
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void setNodeExecutor(String name) throws HeroException {
		 log.debug("setBnNodeExecutor: name=" + name);
		 BnUserLocalHome uHome;
		 BnUserLocal user;
		 try {
			 uHome = BnUserUtil.getLocalHome();
		 } catch (javax.naming.NamingException ne) {
			 throw new HeroException(ne.getMessage());
		 }
		 BnNodeLocal nd = mProject.getBnNode(name);
		 String executor = mContext.getCallerPrincipal().getName();
		 try {
			 user = uHome.findByName(executor);
		 } catch (FinderException fe) {
			 throw new HeroException("BnUser " + executor + " does not exists");
		 }
		 nd.setExecutor(user);
		 jms.sendNodeEvent(SETNODEEXECUTOR,this.getName(),name,nd.getType(),nd.getState(),mContext.getCallerPrincipal().getName());
	 }

	 /**
	  * Set the node type. Change the current type of the node (if node is not executing).<br><br>
	  * 
	  * Possibles node types are:<br>
	  * hero.interfaces.Constants.Nd.AND_JOIN_NODE<br>
	  * hero.interfaces.Constants.Nd.OR_JOIN_NODE<br>
	  * hero.interfaces.Constants.Nd.AND_JOIN_AUTOMATIC_NODE<br>
	  * hero.interfaces.Constants.Nd.OR_JOIN_AUTOMATIC_NODE<br>
	  *
	  * @param name the name of the node
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void setNodeType(String name, int type) throws HeroException {
		 log.debug("setBnNodeType: name=" + name + " type" + type);
		 BnNodeLocal nd = mProject.getBnNode(name);
		 nd.setType(type);
		 nd.setTransition(NodeState.make(nd.getType(), true));
		 nd.setActivation(EdgeState.make(nd.getType()));
	 }

	 /**
	  * Set a property of a node. A property is a pair key/value representing workflow relevant data. By using
	  * propagate argument we can specify if we want to propagate this property.
	  *
	  * @param nodeName the name of the node
	  * @param key the key of the property
	  * @param value the name of the property
	  * @param propagate if it is a propagation property
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void setNodeProperty(String nodeName,String key,String value,boolean propagate)
		 throws HeroException {

		 BnNodePropertyLocalHome pHome;
		 BnNodePropertyLocal propertyLocal;

		 log.debug("setBnNodeProperty node="+ nodeName+ " key="+ key+ " value="+ value);

		 BnNodeLocal nd = mProject.getBnNode(nodeName);

		 try {
			 pHome = BnNodePropertyUtil.getLocalHome();
		 } catch (javax.naming.NamingException ne) {
			 throw new HeroException(ne.getMessage());
		 }
		 Collection c = null;
		 try {
			 c = pHome.findByTheKey(nd.getId(), key);
		 } catch (FinderException fe) {
			 throw new HeroException(fe.getMessage());
		 }
		 if (!c.isEmpty()) {
			 propertyLocal = (BnNodePropertyLocal) (c.toArray())[0];
			 propertyLocal.setTheValue(value);
			 propertyLocal.setPropagate(propagate);
		 } else {
			 try {
				 propertyLocal = pHome.create(key, value);
				 propertyLocal.setBnNode(nd);
				 propertyLocal.setPropagate(propagate);
			 } catch (InvalidValueException ie) {
				 throw new HeroException(ie.getMessage());
			 } catch (javax.ejb.CreateException ce) {
				 throw new HeroException(ce.getMessage());
			 }
		 }
	 }

	 /**
	  * Set a property of a node. A property is a pair key/value representing workflow relevant data. With
	  * this method the property is propagated within others nodes.
	  *
	  * @param nodeName the name of the node
	  * @param key the key of the property
	  * @param value the name of the property
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/

	 public void setNodeProperty(String nodeName, String key, String value)
		 throws HeroException {
		 BnNodePropertyLocalHome pHome;
		 BnNodePropertyLocal propertyLocal = null;

		 log.debug("setBnNodeProperty node="+ nodeName+ " key="+ key+ " value="+ value);
		 BnNodeLocal nd = mProject.getBnNode(nodeName);

		 try {
			 pHome = BnNodePropertyUtil.getLocalHome();
		 } catch (javax.naming.NamingException ne) {
			 throw new HeroException(ne.getMessage());
		 }
		 Collection c = null;
		 try {
			 c = pHome.findByTheKey(nd.getId(), key);
		 } catch (FinderException fe) {
			 throw new HeroException(fe.getMessage());
		 }

		 if (!c.isEmpty()) {

			 propertyLocal = (BnNodePropertyLocal) (c.toArray())[0];
			 propertyLocal.setTheValue(value);

		 } else {
			 try {
				 propertyLocal = pHome.create(key, value);
				 propertyLocal.setBnNode(nd);

			 } catch (InvalidValueException ie) {
				 throw new HeroException(ie.getMessage());
			 } catch (javax.ejb.CreateException ce) {
				 throw new HeroException(ce.getMessage());
			 }
		 }
	 }

	 /**
	  * Set a property of the project. A property is a pair key/value representing workflow relevant data. This
	  * methods creates a new property that can be accessed from all activites of the project.
	  *
	  * @param key the key of the property
	  * @param value the name of the property
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  * @throws HeroException
	  *
	  **/
	 public void setProperty(String key, String value) throws HeroException {
		 log.debug("setProperty key=" + key + " value=" + value);

		 BnProjectPropertyLocalHome pHome;
		 BnProjectPropertyLocal ppLocal = null;

		 try {
			 pHome = BnProjectPropertyUtil.getLocalHome();
		 } catch (javax.naming.NamingException ne) {
			 throw new HeroException(ne.getMessage());
		 }
		 Collection c = null;
		 try {
			 c = pHome.findByTheKey(mProject.getId(), key);
		 } catch (FinderException fe) {
			 throw new HeroException(fe.getMessage());
		 }

		 if (!c.isEmpty()) {

			 ppLocal = (BnProjectPropertyLocal) (c.toArray())[0];
			 ppLocal.setTheValue(value);

		 } else {
			 try {
				 ppLocal = pHome.create(key, value);
				 ppLocal.setBnProject(mProject);

			 } catch (InvalidValueException ie) {
				 throw new HeroException(ie.getMessage());
			 } catch (javax.ejb.CreateException ce) {
				 throw new HeroException(ce.getMessage());
			 }
		 }
	 }

	 /**
	  * Execute a node hook.
	  *
	  * @param nodeName the name of the node
	  * @param event the event 
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Supports"
	  * @throws HeroException
	  *
	  **/
	 public void hookEvent(String nodeName, String event) throws HeroException {
		 log.debug("hookEvent: node=" + nodeName + " event=" + event);
		 Collection hooks;
		 BnNodeLocal node;

		 node = mProject.getBnNode(nodeName);
		 hooks = node.getBnHooks();
		 for (Iterator i = hooks.iterator(); i.hasNext();) {
			 BnNodeHookLocal hk = (BnNodeHookLocal) i.next();
			 if (hk.getEvent().equalsIgnoreCase(event)) {
				 hero.hook.Hook h =
					 hero.hook.Hook.make(hk.getName(), event, hk.getType());
				 h.execute(this, event, node);
			 }
		 }
	 }

	
   
   
   
   // -------------------------------------------------------------------------
   // Basic Get Methods
   // -------------------------------------------------------------------------
  
   /**
   * Get ProjectName
   * @return String 
   *
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Supports"
   *
   **/ 
    public String getName() throws HeroException {
	log.debug("getName:");
	return mProject.getName();
    }

   /**
   * Get creator
   * @return String 
   *
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Supports"
   *
   **/ 
    public String getCreator() throws HeroException {
	log.debug("getCreator:");
	return mProject.getCreator();
    }

   /**
   * Get user roles in project
   * @return Collection 
   *
   * @param userName The name of the user
   *
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Supports"
   *
   **/ 
    public Collection getUserRolesInProject(String userName) throws HeroException {
	log.debug("getUserRolesInProject:");
	BnUserLocalHome uHome;
	BnUserLocal user;
	ArrayList result=new ArrayList();
	try {
	    uHome = (BnUserLocalHome)hero.interfaces.BnUserUtil.getLocalHome();
	} catch (javax.naming.NamingException be) {
	    throw new HeroException(be.getMessage());
	}	
	try {
	    user=uHome.findByName(userName);
	} catch (FinderException f) {
	    throw new HeroException("FinderException");
	}
	Collection userRoles = user.getBnRoles();
        Collection projectRoles=mProject.getBnRoles();
	for(Iterator i=userRoles.iterator();i.hasNext();){
	    BnRoleLocal ur=(BnRoleLocal)i.next();		
	    for(Iterator j=projectRoles.iterator();j.hasNext();){	    
		BnRoleLocal up=(BnRoleLocal)j.next();		    
		if(ur.getId()==up.getId())
		    result.add(ur.getBnRoleValue());
	    }
	}
	return result;
    }
    
   /**
   * Get the node executor
   *
   * @param name of the node
   * @return The user executor
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Supports"
   *
   **/    
    public String getNodeExecutor(String name) throws HeroException {
	log.debug("setNodeDescription: name="+name);
	BnNodeLocal nd=mProject.getBnNode(name);
	BnUserLocal ul = nd.getExecutor();
	return(ul.getName());
    }

    /**
     * Set the node executor
     *
     * @param name of the node
     * @return The state of the node
     * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Supports"
     *
     **/
    public int getNodeState(String name) throws HeroException {
	log.debug("setNodeState: name=" + name);
	BnNodeLocal nd = mProject.getBnNode(name);
	int s = nd.getState();
	return (s);
    }

   /**
     * Get BnNode properties of the project
     *
     * @param nodeName the name of the node
     * @return A BnNodePropertyValue collection of all node properties
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Supports"
     *
     **/            
    public Collection getNodeProperties(String nodeName) throws HeroException {
	Collection prp = new ArrayList();
        BnNodePropertyLocal pl;

	BnNodeLocal nd=mProject.getBnNode(nodeName);

	Collection pls=nd.getBnProperties();

	for(Iterator i=pls.iterator();i.hasNext();){
	    pl=(BnNodePropertyLocal)i.next();
	    prp.add(pl.getBnNodePropertyValue());
	}
	return (prp);
    }

   /**
     * Get BnNode hooks of the project
     *
     * @param nodeName the name of the node
     * @return A BnNodeHookValue collection of all node hooks
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Supports"
     *
     **/            
    public Collection getNodeHooks(String nodeName) throws HeroException {
	Collection hk = new ArrayList();
        BnNodeHookLocal pl;

	BnNodeLocal nd=mProject.getBnNode(nodeName);

	Collection pls=nd.getBnHooks();

	for(Iterator i=pls.iterator();i.hasNext();){
	    pl=(BnNodeHookLocal)i.next();
	    hk.add(pl.getBnNodeHookValue());
	}
	return (hk);
    }
        
   /**
     * Get properties of the project
     * @return A BnProjectPropertyValue collection of all project properties
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Supports"
     *
     **/            
    public Collection getProperties() throws HeroException {
	Collection prp = new ArrayList();
	BnProjectPropertyLocal ppl;
	
	Collection pls=mProject.getBnProperties();
	for(Iterator i=pls.iterator();i.hasNext();){
	    ppl=(BnProjectPropertyLocal)i.next();
	    prp.add(ppl.getBnProjectPropertyValue());
	}
	return (prp);
    }

   /**
     * Get hooks of the project
     * @return A ProjectHooksValue collection of all project hook
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Supports"
     *
     **/            
    public Collection getHooks() throws HeroException {
	Collection hooks = new ArrayList();
	BnProjectHookLocal phl;
	
	Collection hks=mProject.getBnHooks();
	for(Iterator i=hks.iterator();i.hasNext();){
	    phl=(BnProjectHookLocal)i.next();
	    hooks.add(phl.getBnProjectHookValue());
	}
	return (hooks);
    }

   /**
   * Get all users of the project
   * @return Collection 
   *
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Supports"
   *
   **/ 
    public Collection getUsers() throws HeroException {
	log.debug("getUsers:");
	Collection users = mProject.getBnUsers();
	ArrayList result=new ArrayList();
	for (Iterator i=users.iterator();i.hasNext();) {
	    result.add(((BnUserLocal)i.next()).getName());
	}
	return result;
    }

   /**
   * Return all registered users
   * @return A string collection of all user names
   *
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Supports"
   *
   **/ 
    public Collection getAllUsers() throws HeroException {
	log.debug("getAllUsers:");
	BnUserLocalHome uHome;
	Collection users;
	try {
	    uHome = (BnUserLocalHome)hero.interfaces.BnUserUtil.getLocalHome();
	} catch (javax.naming.NamingException be) {
	    throw new HeroException(be.getMessage());
	}	
	try {
	    users=uHome.findAll();
	} catch (FinderException f) {
	    throw new HeroException("FinderException in findByAll users");
	}
	ArrayList result=new ArrayList();
	for (Iterator i=users.iterator();i.hasNext();) {
	    result.add(((BnUserLocal)i.next()).getName());
	}
	return result;
    }

   /**
   * Get all role names of the project
   * @return Collection 
   *
   * @ejb:interface-method view-type="both
   * @ejb:transaction type="Supports"
   *
   **/ 
    public Collection getRolesNames() throws HeroException {
	log.debug("getRolesNames:");
	Collection roles = mProject.getBnRoles();
	ArrayList result=new ArrayList();
	for (Iterator i=roles.iterator();i.hasNext();) {
	    result.add(((BnRoleLocal)i.next()).getName());
	}
	return result;
    }

    /**
     * Get all node names of the project
     *
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Supports"
     *
     **/       
    public Collection getNodesNames() throws HeroException {
	log.debug("getNodesNames: ");

	Vector names = new Vector();
	Collection nodes = mProject.getBnNodes();
	Iterator i = nodes.iterator();
	while (i.hasNext())
	{
	    BnNodeLocal n = (BnNodeLocal)i.next();
	    names.add(n.getName());
	}
	return((Collection)names);
    }

    /**
     * Get all node  of the project
     *
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Supports"
     *
     **/       
    public Collection getNodes() throws HeroException {
	log.debug("getNodes: ");
	Vector nodesDetails = new Vector();
	Collection nodes = mProject.getBnNodes();
	
	Iterator i = nodes.iterator();
	while (i.hasNext())
	{
	    BnNodeLocal n = (BnNodeLocal)i.next();
	    String[] details = new String[5];
	    details[0]=n.getName();
	    details[1]=hero.interfaces.Constants.Nd.nodeStateName[n.getState()];
	    details[2]=((BnRoleLocal)n.getBnRole()).getName();
	    if (n.getDeadline()!=null)
		details[3]=n.getDeadline().toString();
	    else
		details[3]="null";
	    details[4]=(new Boolean(n.getAnticipable())).toString();
	    nodesDetails.add(details);
	}
	return((Collection)nodesDetails);
    }

    /**
     * Get all node information
     *
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Supports"
     *
     **/       

    public Collection getNodeDetails(String node) throws HeroException {

	BnNodeLocal nd=mProject.getBnNode(node);
	Vector detail = new Vector();

	detail.add(nd.getName()); 
	detail.add(mProject.getName());
	detail.add(((BnUserLocal)nd.getCreator()).getName());
	detail.add(hero.interfaces.Constants.Nd.nodeStateName[nd.getState()]);
       	detail.add(((BnRoleLocal)nd.getBnRole()).getName());
	if ((BnUserLocal)nd.getExecutor()!=null)
	    detail.add(((BnUserLocal)nd.getExecutor()).getName());
	else
	    detail.add("null");

	if (nd.getDeadline()!=null)	
	    detail.add(nd.getDeadline().toString());
	else detail.add("null");

    	detail.add(new Boolean((nd.getAnticipable())).toString());

	if (nd.getDescription()!=null)
	    detail.add(nd.getDescription());
	else  detail.add("null");
	
	return((Collection)detail);
    }

   /**
   * Get user roles in project
   * @return Collection 
   *
   * @param userName The name of the user
   *
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Supports"
   *
   **/ 
    public Collection getUserRolesInProjectNames(String userName) throws HeroException {
	log.debug("getUserRolesInProject:");
	BnUserLocalHome uHome;
	BnUserLocal user;
	ArrayList result=new ArrayList();
	try {
	    uHome = (BnUserLocalHome)hero.interfaces.BnUserUtil.getLocalHome();
	} catch (javax.naming.NamingException be) {
	    throw new HeroException(be.getMessage());
	}	
	try {
	    user=uHome.findByName(userName);
	} catch (FinderException f) {
	    throw new HeroException("FinderException");
	}
	Collection userRoles = user.getBnRoles();
        Collection projectRoles=mProject.getBnRoles();
	for(Iterator i=userRoles.iterator();i.hasNext();){
	    BnRoleLocal ur=(BnRoleLocal)i.next();		
	    for(Iterator j=projectRoles.iterator();j.hasNext();){	    
		BnRoleLocal up=(BnRoleLocal)j.next();		    
		if(ur.getId()==up.getId())
		    result.add(ur.getName());
	    }
	}
	return result;
    }

   /**
   * get all in edges of the node
   * @return Collection 
   *
   * @param name the name ot the node
   *
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Supports"
   *
   **/        
    public Collection getNodeInEdges(String name) throws HeroException {
	log.debug("getNodeInEdges: name="+name);
	
	Vector in=new Vector();
	BnNodeLocal nd=mProject.getBnNode(name);
	
	Collection edgesIn=nd.getInBnEdges();
	Iterator i = edgesIn.iterator();
	while (i.hasNext()){
	    BnEdgeLocal e = (BnEdgeLocal)i.next();
	    String[] details = new String[4];
	    details[0]=e.getName();
	    details[1]=hero.interfaces.Constants.Ed.edgeStateName[e.getState()];
	    BnNodeLocal nIn = (BnNodeLocal)e.getInBnNode();
	    details[2]=nIn.getName();
	    BnNodeLocal nOut = (BnNodeLocal)e.getOutBnNode();
	    details[3]=nOut.getName();
	    in.add(details);
	}
	
	return(in);
    }
     
    /**
     * Get all BnEdge  of the project
     *
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Supports"
     *
     **/       
    public Collection getEdges() throws HeroException {
	log.debug("getEdges: ");
	Vector edgesDetails = new Vector();
	Collection edges = mProject.getBnEdges();
	Iterator i = edges.iterator();
	while (i.hasNext())
	{
	    BnEdgeLocal e = (BnEdgeLocal)i.next();
	    String[] details = new String[4];
	    details[0]=e.getName();
	    details[1]=hero.interfaces.Constants.Ed.edgeStateName[e.getState()];
	    BnNodeLocal nIn = (BnNodeLocal)e.getInBnNode();
	    details[2]=nIn.getName();
	    BnNodeLocal nOut = (BnNodeLocal)e.getOutBnNode();
	    details[3]=nOut.getName();
	    edgesDetails.add(details);
	}
	return((Collection)edgesDetails);
    }
	    
   /**
   * get all out edges of the node
   * @return Collection 
   *
   * @param name the name ot the node
   *
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Supports"
   *
   **/        
    public Collection getNodeOutEdges(String name) throws HeroException {
	log.debug("getNodeOutEdges: name="+name);

	Vector out=new Vector();
	BnNodeLocal nd=mProject.getBnNode(name);

	Collection edgesOut=nd.getOutBnEdges();
	Iterator i = edgesOut.iterator();
	while (i.hasNext()){
	    BnEdgeLocal e = (BnEdgeLocal)i.next();
	    String[] details = new String[4];
	    details[0]=e.getName();
	    details[1]=hero.interfaces.Constants.Ed.edgeStateName[e.getState()];
	    BnNodeLocal nIn = (BnNodeLocal)e.getInBnNode();
	    details[2]=nIn.getName();
	    BnNodeLocal nOut = (BnNodeLocal)e.getOutBnNode();
	    details[3]=nOut.getName();
	    out.add(details);
	}
	
	return(out);
    }

	/**
	 * Get the type of the node
	 *
	 * @param name of the node
	 * @return The type of the node
	 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Supports"
	 *
	 **/
	public int getNodeType(String name) throws HeroException {
		log.debug("setBnNodeType: name=" + name);
		BnNodeLocal nd = mProject.getBnNode(name);
		return (nd.getType());
	}

	/**
	 * Get the ProjectValue (historique). Returns project information: project attributes, nodes, edges,
	 * hooks, properties...
	 *
	 * @return BnProjectValue object - project data
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public BnProjectValue getDetails() throws HeroException {
		log.debug("getDetails:");
		return mProject.getBnProjectValue();
	}
	
	/**
	 * Import all the nodes, hooks and edges of a project to the current project
	 *
	 * @param projectname the name of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void importProject(String projectName) throws HeroException {
		log.debug("importProject: project=" + projectName);
		importProject(projectName, "");
	}

	// -------------------------------------------------------------------------
	// ejbCreate and initProject business methods
	// -------------------------------------------------------------------------

	/**
	 * Import all the nodes, hooks, properties and edges of a project to the current project
	 *
	 * @param projectname the name of the project
	 * @param prefix the prefix of names for the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void importProject(String projectName, String prefix)
		throws HeroException {
		log.debug("importProject: project=" + projectName + " prefix=" + prefix);
		try {
			BnProjectLocalHome opHome =	hero.interfaces.BnProjectUtil.getLocalHome();
			BnProjectLocal oldProject = opHome.findByName(projectName);
			String actualProject = this.getName();

			// Project Properties (not initialized)
			 Collection props = oldProject.getBnProperties();
			 for (Iterator i = props.iterator(); i.hasNext();) {
				 BnProjectPropertyLocal prop = (BnProjectPropertyLocal) i.next();
				 this.setProperty(prop.getTheKey(),"");
			 }

			//Project InterHooks
			Collection hooks = oldProject.getBnInterHooks();
			for (Iterator i = hooks.iterator(); i.hasNext();) {
				BnProjectInterHookLocal hk = (BnProjectInterHookLocal) i.next();
				this.addInterHook(hk.getName(),hk.getEvent(),hk.getType(),hk.getScript());
			}

			Collection roles = oldProject.getBnRoles();
			Iterator r = roles.iterator();
			while (r.hasNext())
			{
				BnRoleLocal ro = (BnRoleLocal)r.next();
				String roleName = ro.getName();
				if (!containsRole(roleName)) {
					this.addRole(roleName, ro.getDescription());
				}
			} 

			//	Copy not connected nodes
			Collection allNodes = oldProject.getBnNodes();
			Iterator no = allNodes.iterator();
			while (no.hasNext()) {
				BnNodeLocal node = (BnNodeLocal) no.next();
				if (node.getInBnEdges().isEmpty()&& node.getOutBnEdges().isEmpty()) {
				// New Instance if activity is a subProcess
					String newNodeName=node.getName();
					if (node.getType() == hero.interfaces.Constants.Nd.SUB_PROCESS_NODE) {
						Iterator pr = subProjects.iterator();
						boolean exit = false;
						while (pr.hasNext()) {
						String nodePr = (String) pr.next();
						if (nodePr.equals(newNodeName)) {
							newNodeName = actualProject + nodePr;
							exit = true;
						}
					}
					try {
						if (!exit) {
						  this.initProject(newNodeName, actualProject + newNodeName);
						  subProjects.add(newNodeName);
						  newNodeName = actualProject + newNodeName;
						  mProject = opHome.findByName(actualProject);
						}
					} catch (Exception ce) {
						ce.printStackTrace();
					}			
				}			
					
					this.addNode(newNodeName, node.getType());
					if (!node.getAnticipable())
						this.setNodeTraditional(newNodeName);
					this.setNodeRole(newNodeName,node.getBnRole().getName());
					this.setNodeDescription(newNodeName, node.getDescription());

					//Node InterHooks
					Collection nodeHooks = node.getBnInterHooks();
					for (Iterator j = nodeHooks.iterator(); j.hasNext();) {
						BnNodeInterHookLocal nh = (BnNodeInterHookLocal) j.next();
						try {
							this.addNodeInterHook(newNodeName,nh.getName(),nh.getEvent(),nh.getType(),nh.getScript());
						} catch (Exception ih) {
						}; // For subProcess Instance consistency
					}
				}

			}
			
			Collection edges = oldProject.getBnEdges();
			Iterator e = edges.iterator();
			while (e.hasNext())
			{
				BnEdgeLocal ed = (BnEdgeLocal)e.next();
				BnNodeLocal nodeIn = ed.getInBnNode();
				BnNodeLocal nodeOut = ed.getOutBnNode();
				BnNodeLocal[] nodes = { nodeIn, nodeOut };
				String newNodeNameIn = null;
				String newNodeNameOut = null;
				for (int n = 0; n < nodes.length; n++) {
					String newNodeName = nodes[n].getName();
					// New Instance if activity is a subProcess
					if (nodes[n].getType()== hero.interfaces.Constants.Nd.SUB_PROCESS_NODE){
						Iterator pr = subProjects.iterator();
						boolean exit = false;
						while (pr.hasNext()) {
							String nodePr = (String) pr.next();
							if (nodePr.equals(newNodeName)) {
								newNodeName = actualProject+nodePr;
								exit = true;
							}
						}
						try {
						if (!exit) {
							this.initProject(newNodeName,actualProject+newNodeName);
							subProjects.add(newNodeName);
							newNodeName=actualProject+newNodeName;
							mProject = opHome.findByName(actualProject);
						}
						}catch(Exception ce){ce.printStackTrace();}
					}
					try {
						this.addNode(newNodeName, nodes[n].getType());
						if (!nodes[n].getAnticipable())
							this.setNodeTraditional(newNodeName);
						this.setNodeRole(newNodeName,nodes[n].getBnRole().getName());
						this.setNodeDescription(newNodeName,nodes[n].getDescription());
						// Node iteration properties
						Collection nodeProps = oldProject.getBnNode(nodes[n].getName()).getBnProperties();
						for (Iterator i = nodeProps.iterator(); i.hasNext();) {
							 BnNodePropertyLocal prop = (BnNodePropertyLocal) i.next();
							 if (prop.getTheKey().equals("iterate"))
								this.setNodeProperty(newNodeName,prop.getTheKey(),prop.getTheValue());
						 }

						//Node InterHooks
						BnNodeLocal nd = oldProject.getBnNode(nodes[n].getName());
						Collection nodeHooks = nd.getBnInterHooks();
						for (Iterator j = nodeHooks.iterator(); j.hasNext();) {
							BnNodeInterHookLocal nh = (BnNodeInterHookLocal) j.next();
							try {
								this.addNodeInterHook(newNodeName,nh.getName(),nh.getEvent(),nh.getType(),nh.getScript());
							} catch (Exception ih) {
							}; // For subProcess Instance consistency
						}
					} catch (Exception an) {
					}; // Node already exist
					if (n == 0)
						newNodeNameIn = newNodeName;
					else
						newNodeNameOut = newNodeName;
				}

				String edgeName = this.addEdge(newNodeNameIn, newNodeNameOut);
				this.setEdgeCondition(edgeName, ed.getCondition());
			}

		} catch (FinderException e) {
			throw new HeroException("Cannot find project " + e.getMessage());
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
	}

	/**
	 * Import Project instance. Import all the nodes, hooks, properties,edges and users of a project instant to the
	 * current project
	 *
	 * @param projectname the name of the project
	 * @param instance the instance name of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void importInstance(String projectName, String instance)
		throws HeroException {
		log.debug("ImportInstance: project=" + projectName);
		try {
			BnProjectLocalHome opHome =	hero.interfaces.BnProjectUtil.getLocalHome();
			BnProjectLocal oldProject = opHome.findByName(projectName);
			
			BnProjectValue pv = oldProject.getBnProjectValue();
			

			// Project Properties (not initialized)

			BnProjectPropertyValue[] props = pv.getBnProperties();
			Collection insertProp = new ArrayList();
			int ppIndex = 0;
			for (ppIndex=0;ppIndex<props.length;ppIndex++)
			{
				BnProjectPropertyValue prop = props[ppIndex];
				this.setProperty(props[ppIndex].getTheKey(),"");
				
			}

			BnProjectInterHookValue[] hooks = pv.getBnInterHooks();
			Collection insertHooks = new ArrayList();
			int pihIndex = 0;
			for (pihIndex=0;pihIndex<hooks.length;pihIndex++)
			{
				BnProjectInterHookValue hk = hooks[pihIndex];
				this.addInterHook(hk.getName(),hk.getEvent(),hk.getType(),hk.getScript());
			}
			
			BnRoleValue[] roles = pv.getBnRoles();
			Collection insertRoles = new ArrayList();
			int rIndex = 0;
			for (rIndex=0;rIndex<roles.length;rIndex++)
			{
				BnRoleValue ro = roles[rIndex];
				String roleName = ro.getName();
				if (!containsRole(roleName))
					this.addRole(roleName, ro.getDescription());
				
			}
			
			// Copy not connected nodes
			
			BnNodeValue[] allNodes = pv.getBnNodes();
			//Collection insertNodes = new ArrayList();
			int nIndex = 0;
			for (nIndex=0;nIndex<allNodes.length;nIndex++)
			{
				Collection insertNodeHooks = new ArrayList();
				BnNodeValue node = allNodes[nIndex];
				BnNodeLocal nodeLocal = oldProject.getBnNode(node.getName());
				if (nodeLocal.getInBnEdges().size() == 0 && nodeLocal.getOutBnEdges().size()==0)
				{
					String newNodeName=node.getName();
					if (node.getType()== hero.interfaces.Constants.Nd.SUB_PROCESS_NODE && !subProjects.contains(node.getName())) {
						Iterator pr = subProjects.iterator();
						boolean exit=false;
						while (pr.hasNext())
						{
						  String nodePr = (String)pr.next();
						  if (nodePr.matches(newNodeName+"_instance"+".*"))
						  {
							  newNodeName = nodePr;
							  exit = true;
						  }	
						 }
						 if (!exit)
						 {
							newNodeName = this.instantiateProject(newNodeName);
							subProjects.add(newNodeName);
							mProject.setParent(instance);
							mProject = opHome.findByName(instance);
						 }
					}
					try{					
					  this.addNode(newNodeName, node.getType());
					  if (!node.getAnticipable())
						this.setNodeTraditional(newNodeName);
					  this.setNodeRole(newNodeName,node.getBnRole().getName());
					  this.setNodeDescription(newNodeName,node.getDescription());
					  //Node InterHooks
					}catch(Exception ne){}
					BnNodeInterHookValue[] nodeHooks = node.getBnInterHooks();
					int niIndex = 0;
					for (niIndex=0;niIndex<nodeHooks.length;niIndex++)
					{
						BnNodeInterHookValue nh = nodeHooks[niIndex];
						try {
							this.addNodeInterHook(newNodeName,nh.getName(),nh.getEvent(),nh.getType(),nh.getScript());
						} catch (Exception ih) {
						}; // For subProcess Instance consistency
					}
				}
			}
				
			// Copy connected nodes and edges
			
			BnEdgeValue[] edges = pv.getBnEdges();
			
			int eIndex = 0;
			for (eIndex=0;eIndex<edges.length;eIndex++)
			{
				BnEdgeValue ed = edges[eIndex];
				BnNodeValue nodeIn = ed.getInBnNode();
				BnNodeValue nodeOut = ed.getOutBnNode();
				BnNodeValue[] nodes = { nodeIn, nodeOut };
				String newNodeNameIn = null;
				String newNodeNameOut = null;
				for (int n = 0; n < nodes.length; n++) {
					String newNodeName = nodes[n].getName();
					// New Instance if activity is a subProcess
					if (nodes[n].getType()== hero.interfaces.Constants.Nd.SUB_PROCESS_NODE && !subProjects.contains(nodes[n].getName())) {
						Iterator pr = subProjects.iterator();
						boolean exit=false;
						while (pr.hasNext())
						{
							String nodePr = (String)pr.next();
							if (nodePr.matches(newNodeName+"_instance"+".*"))
							{
							  newNodeName = nodePr;
							  exit = true;
							}	
						}
						if (!exit)
						{
							newNodeName = this.instantiateProject(newNodeName);
							subProjects.add(newNodeName);
							mProject.setParent(instance);
							mProject = opHome.findByName(instance);
						}
					}
					try{					  	
					  this.addNode(newNodeName, nodes[n].getType());
					  if (!nodes[n].getAnticipable())
						  this.setNodeTraditional(newNodeName);
					  this.setNodeRole(newNodeName,nodes[n].getBnRole().getName());
					  this.setNodeDescription(newNodeName,nodes[n].getDescription());			  
					 					  
					  BnNodePropertyValue[] nodeProps = nodes[n].getBnProperties();
					  int nopIndex=0;
					  for (nopIndex=0;nopIndex<nodeProps.length;nopIndex++)
					  {
						BnNodePropertyValue prop = nodeProps[nopIndex];
						if (prop.getTheKey().equals("iterate"))
						   this.setNodeProperty(newNodeName,prop.getTheKey(),prop.getTheValue());
						else  	
						   this.setNodeProperty(newNodeName,prop.getTheKey(),"");
					  }
					 
					  //Node InterHooks
					  BnNodeInterHookValue[] nodeHooks = nodes[n].getBnInterHooks();
					  int nihIndex=0;
					  for (nihIndex=0;nihIndex<nodeHooks.length;nihIndex++)
					  {
						BnNodeInterHookValue nh = nodeHooks[nihIndex];
						try {
						 this.addNodeInterHook(newNodeName,nh.getName(),nh.getEvent(),nh.getType(),nh.getScript());
						} catch (Exception ih) {
						 };  // For subProcess Instance consistency
					  }
					}catch(Exception an){}; // Node already exist
					if (n == 0)
						newNodeNameIn = newNodeName;
					else
						newNodeNameOut = newNodeName;
				}

				String edgeName = this.addEdge(newNodeNameIn, newNodeNameOut);
				this.setEdgeCondition(edgeName,ed.getCondition());
			}
			//pvInsert.setAddedBnNodes(insertNodes);
			
			BnUserLightValue[] users = pv.getBnUsers();
			int uIndex = 0;
			for (uIndex=0;uIndex<users.length;uIndex++)
			{
				BnUserLightValue userP = users[uIndex];
				try {
				   this.addUser(userP.getName());
				} catch (Exception eUser) {
				jms.sendProjectEvent(ADDUSERPROJECT,mProject.getName(),userP.getName());
				}
				mProject = oldProject;
				Collection userRoles = this.getUserRolesInProjectNames(userP.getName());
				mProject = opHome.findByName(instance);
				Iterator ir = userRoles.iterator();
				while (ir.hasNext())
					this.setUserRole(userP.getName(),(String) ir.next());
			}

		} catch (FinderException e) {
			throw new HeroException("Cannot find project " + e.getMessage());
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (CreateException ce) {
			throw new HeroException("Instanciate SubProcess error" + ce.getMessage());
		}
	}

	/**
	 * Init the Project Session Bean. You have to call this method after "create" call. After that
	 * all methods of ProjectSession API are available.
	 *
	 * @param projectName project name 
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 **/
	public void initProject(String projectName) throws HeroException {
		Context lContext;
		BnUserLocalHome uhome = null;
		BnRoleLocalHome rhome = null;
		log.debug("initProject: project=" + projectName);
		  try {
			pHome = hero.interfaces.BnProjectUtil.getLocalHome();
			uhome = hero.interfaces.BnUserUtil.getLocalHome();
			rhome = hero.interfaces.BnRoleUtil.getLocalHome();
			this.mProject = pHome.findByName(projectName);
		  } catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		  } catch (javax.ejb.FinderException fe) {
			// The project has to be created
			try {
				String creator = mContext.getCallerPrincipal().getName();
				mProject = pHome.create(projectName, creator);
				mProject.setInstanceNs(0);
				mProject.setParent(projectName);
				BnUserLocal user = uhome.findByName(creator);
				mProject.getBnUsers().add(user);
				log.debug("Add user " + user.getBnUserLightValue().toString());
				// role admin is added
				BnRoleValue rv = new BnRoleValue();
				rv.setName(hero.interfaces.Constants.ADMIN);
				rv.setDescription("the admin role");
				BnRoleLocal role = rhome.create(rv);
				mProject.getBnRoles().add(role);
				this.setUserRole(creator, hero.interfaces.Constants.ADMIN);
				// role InitialRole is added
				rv = new BnRoleValue();
				rv.setName(hero.interfaces.Constants.INITIALROLE);
				rv.setDescription("the initial role for nodes");
				role = rhome.create(rv);
				mProject.getBnRoles().add(role);
				jms.sendProjectEvent(ADDUSERPROJECT, projectName, creator);
			 } catch (FinderException fie) {
				throw new HeroException(fie.getMessage());
			 } catch (InvalidValueException ie) {
				throw new HeroException(ie.getMessage());
			 } catch (Exception he) {
				throw new HeroException(he.getMessage());
			 }
		   }
	}

	/**
	 * init the Project Session Bean (clone project).You have to call this method after "create" call. After that
	 * the project is cloned and all methods of ProjectSession API are available.
	 *
	 * @throws HeroException 
	 * @param oldProject the name of the project to clone
	 * @param newProject the name of the project clone.
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws CreateException
	 **/
	public void initProject(String oldProject, String newProject)
		throws CreateException {
		log.debug("initProject: oldProject="+ oldProject+ " newProject="+ newProject);
		Context lContext;
		BnUserLocalHome uhome = null;
		BnRoleLocalHome rhome = null;
		  try {
			pHome = hero.interfaces.BnProjectUtil.getLocalHome();
			uhome = hero.interfaces.BnUserUtil.getLocalHome();
			rhome = hero.interfaces.BnRoleUtil.getLocalHome();
			mProject = pHome.findByName(newProject);
		   } catch (javax.naming.NamingException ne) {
			throw new CreateException(ne.getMessage());
		   } catch (javax.ejb.FinderException fe) {
			// The project has to be created
			try {
				String creator = mContext.getCallerPrincipal().getName();
				mProject = pHome.create(newProject, creator);
				mProject.setInstanceNs(0);
				mProject.setParent(newProject);
				BnUserLocal user = uhome.findByName(creator);
				mProject.getBnUsers().add(user);
				
				log.debug("Add user " + user.getBnUserLightValue().toString());
				// role admin is added
				BnRoleValue rv = new BnRoleValue();
				rv.setName(hero.interfaces.Constants.ADMIN);
				rv.setDescription("the admin role");
				BnRoleLocal role = rhome.create(rv);
				mProject.getBnRoles().add(role);
				this.setUserRole(creator, hero.interfaces.Constants.ADMIN);
				// role InitialRole is added
				rv = new BnRoleValue();
				rv.setName(hero.interfaces.Constants.INITIALROLE);
				rv.setDescription("the initial role for nodes");
				role = rhome.create(rv);
				mProject.getBnRoles().add(role);
				importProject(oldProject);

				jms.sendProjectEvent(ADDUSERPROJECT, newProject, creator);

			  } catch (FinderException fie) {
				throw new CreateException(fie.getMessage());
			  } catch (InvalidValueException ie) {
				throw new CreateException(ie.getMessage());
			  } catch (HeroException he) {
				throw new CreateException(he.getMessage());
			  }
		  }
	}

	/**
	 * Project instance. You have to call this method after "create" call. After that
	 * you have an instance of the project and all methods of ProjectSession API are available.
	 *
	 * @param project the name of the project to instance
	 * @return String - the name of the project instance
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws CreateException 
	 **/
	public String instantiateProject(String project) throws CreateException {
		log.debug("instatiateProject: project=" + project);
		Context lContext;
		BnUserLocalHome uhome = null;
		BnRoleLocalHome rhome = null;
		BnProjectLocal projectModel = null;
		String instance = project;
		try {
			pHome = hero.interfaces.BnProjectUtil.getLocalHome();
			projectModel = pHome.findByName(project);
			int number = projectModel.getInstanceNs() + 1;
			instance = project + "_instance" + number;
			uhome = hero.interfaces.BnUserUtil.getLocalHome();
			rhome = hero.interfaces.BnRoleUtil.getLocalHome();
			mProject = pHome.findByName(instance);
		} catch (javax.naming.NamingException ne) {
			throw new CreateException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			// The project has to be created
			try {
				String creator = mContext.getCallerPrincipal().getName();
				mProject = pHome.create(instance, creator);
				mProject.setParent(instance);
				BnUserLocal user = uhome.findByName(creator);
				mProject.getBnUsers().add(user);
				log.debug("Add user " + user.getBnUserLightValue().toString());
				// role admin is added
				BnRoleValue rv = new BnRoleValue();
				rv.setName(hero.interfaces.Constants.ADMIN);
				rv.setDescription("the admin role");
				BnRoleLocal role = rhome.create(rv);
				mProject.getBnRoles().add(role);
				this.setUserRole(creator, hero.interfaces.Constants.ADMIN);
				// role InitialRole is added
				rv = new BnRoleValue();
				rv.setName(hero.interfaces.Constants.INITIALROLE);
				rv.setDescription("the initial role for nodes");
				role = rhome.create(rv);
				mProject.getBnRoles().add(role);
				importInstance(project, instance);
				projectModel.setInstanceNs(projectModel.getInstanceNs() + 1);

				jms.sendProjectEvent(ADDUSERPROJECT, instance, creator);

			} catch (FinderException fie) {
				throw new CreateException(fie.getMessage());
			} catch (InvalidValueException ie) {
				throw new CreateException(ie.getMessage());
			} catch (HeroException he) {
				throw new CreateException(he.getMessage());
			}
		}
		return (instance);
	}

	/**
	 * Creates the Project Session Bean. This method is the first one to invoke in order to
	 * use ProjectSession API. If the user is not authorized this method throws an exception.
	 *
	 * @throws CreateException 
	 *
	 * @ejb:create-method view-type="both"
	 **/
	public void ejbCreate() throws CreateException {
	}

	/**
	 * Internal Enterprise Java Beans method. 
	**/
	public void ejbPostCreate(String oldProject, String newProject)
		throws CreateException, RemoteException {
	}

	/**
	 * Internal Enterprise Java Beans method. 
	**/
	public void setSessionContext(final javax.ejb.SessionContext context) {
		mContext = context;
		try{
		  JMSServicesSessionLocalHome home = JMSServicesSessionUtil.getLocalHome();
		  this.jms=home.create();
		}catch(Exception e){throw new EJBException(e.getMessage());}
	}

	/**
	 * Internal Enterprise Java Beans method. 
	**/

	public void ejbRemove() {
	}

	/**
	 * Internal Enterprise Java Beans method. 
	**/

	public void ejbActivate() {
	}

	/**
	 * Internal Enterprise Java Beans method. 
	**/

	public void ejbPassivate() {
	}

	// Utility methods

	private boolean checkDynamics(String out) throws HeroException {
		BnNodeLocal nodeout = mProject.getBnNode(out);
		if ((nodeout.getState() == hero.interfaces.Constants.Nd.INITIAL)
			|| (nodeout.getState() == hero.interfaces.Constants.Nd.READY)
			|| (nodeout.getState() == hero.interfaces.Constants.Nd.ANTICIPABLE))
			return true;
		return false;
	}

	private boolean checkStructuralConstraints(String in, String out) throws HeroException {
		visited.add(in);
		BnNodeLocal nodeout = mProject.getBnNode(out);
		if (visited.contains(out))
		  return false;
		Collection edges = nodeout.getOutBnEdges();
		Iterator i = edges.iterator();
		while (i.hasNext())
		{
		  BnEdgeLocal edge = (BnEdgeLocal)i.next();
		  if (!visited.contains(edge.getInBnNode().getName()) && ! checkStructuralConstraints(edge.getInBnNode().getName(),edge.getOutBnNode().getName()))
			return false;
		}
		
		return true;
	}

	private boolean checkDuplicateEdge(String in, String out)
		throws HeroException {
		Collection edges = mProject.getBnEdges();
		Iterator i = edges.iterator();
		while (i.hasNext()) {
			BnEdgeLocal edge = (BnEdgeLocal) i.next();
			if ((edge.getInBnNode().getName()).equals(in)
				&& (edge.getOutBnNode().getName()).equals(out))
				return false;
		}
		return true;
	}

	private boolean checkDuplicateAgentEdge(String in, String out)
		throws HeroException {
		Collection edges = mProject.getBnAgentEdges();
		Iterator i = edges.iterator();
		while (i.hasNext()) {
			BnAgentEdgeLocal edge = (BnAgentEdgeLocal) i.next();
			if ((edge.getInBnAgent().getName()).equals(in)
				&& (edge.getOutBnNode().getName()).equals(out))
				return false;
		}
		return true;
	}

	/**
	* Test if the project contains this user
	* @param userName The name of the user to associate to the project
	* @return boolean - true is the user belongs to the project
	*
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	* @throws HeroException
	*
	**/
	public boolean containsUser(String userName) throws HeroException {
		log.debug("containsUser : name=" + userName);
		BnUserLightValue[] uv = mProject.getBnProjectValue().getBnUsers();
			for (int i = 0; i < uv.length; i++) {
				if (uv[i].getName().equalsIgnoreCase(userName))
					return true;
			}
			return false;
		}

		
		/*BnUserLocalHome uHome;
		BnUserLocal mUser;

		try {
			uHome = (BnUserLocalHome) BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			mUser = uHome.findByName(userName);
		} catch (javax.ejb.FinderException fe) {
			throw new HeroException("BnUser does not exists");
		}
		Collection cUsers = mProject.getBnUsers();
		return (cUsers.contains(mUser));
	}*/

	private boolean containsRole(String roleName) {
		BnRoleValue[] rv = mProject.getBnProjectValue().getBnRoles();
		for (int i = 0; i < rv.length; i++) {
			if (rv[i].getName().equalsIgnoreCase(roleName))
				return true;
		}
		return false;
	}

	private void removeOldNotification(String name) throws HeroException {
		try {
			ObjectName timer = new ObjectName("bonita.jmx:service=timer");
			List list = MBeanServerFactory.findMBeanServer(null);
			MBeanServer server = (MBeanServer) list.iterator().next();

			server.invoke(timer,"removeNotifications",new Object[] { name },new String[] { "".getClass().getName()});
		} catch (javax.management.InstanceNotFoundException ex) {
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}
	


}
